classdef getAUTOSARProperties < handle
% autosar.api.getAUTOSARProperties API class.
%
% DATAOBJ = autosar.api.getAUTOSARProperties(MDLNAME) creates an object
% DATAOBJ that can be used to query and set the AUTOSAR Properties of 
% Simulink model MDLNAME. 
%
% getAUTOSARProperties methods:
%    add                      - add a property to an AUTOSAR element
%    addPackageableElement    - add an AUTOSAR element to an AUTOSAR package
%    delete                   - delete an AUTOSAR element
%    find                     - find AUTOSAR elements  
%    get                      - get the property of an AUTOSAR element
%    set                      - set the property of an AUTOSAR element
%    deleteUnmappedComponents - delete all unmapped components
%
% See also autosar.api.create autosar.api.getSimulinkMapping
    
%   Copyright 2012-2016 The MathWorks, Inc.

    properties(Access=private, Transient=true)
        ModelName;
        IsStrict; % Enforce strict parameter checking
    end
    
    properties(Constant, Access=private)
        UnsupportedCategories =  {'Boolean', 'Complex', ...
            'ConstantSpecification', 'DataConstr', 'Dependency',...
            'Enumeration', 'FixedPoint', 'FloatingPoint',...
            'Integer', 'Matrix', 'Structure', 'VoidPointer',...
            'LookupTableType', 'SharedAxisType'};
    end
    
    methods(Access=public)
        function this = getAUTOSARProperties(modelName, isStrict)
            this.ModelName = modelName;
            autosar.api.Utils.convertAI2MM(modelName);
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            autosar.ui.utils.registerListenerCB(m3iModel);
            if nargin < 2
                isStrict = false;
            end
            this.IsStrict = isStrict;
        end

        function paths = find(this, rootPath, category, varargin)
            % PATHS = FIND(THIS, ROOTPATH, CATEGORY, P1, V1, ...) is a cell array containing
            % the PATHS to the AUTOSAR elements matching the type CATEGORY
            % starting at path ROOTPATH. Constraints may be imposed on the
            % property values of the element of interest using Name-Value pairs
            % P1/V1. An optional parameter 'PathType' can be used to specify
            % whether the PATHS are 'FullyQualified' or 'PartiallyQualified'.
            %
            % Examples:
            % Find all the SenderReceiver interfaces with property IsService set
            % to false
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % ifPaths = dataObj.find([], 'SenderReceiverInterface', ...
            % 'IsService', false, 'PathType', 'FullyQualified');
            %
            % See also add, delete, get, set

            % Validate method arguments
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            paths = this.find_impl(m3iModel, rootPath, category, varargin{:});            
        end
    
        function paramValue = get(this, elementPath, propertyName, varargin)
            % PARAMVALUE = GET(DATAOBJ, ELEMENTPATH, PROPERTYNAME) is the value of the
            % property of AUTOSAR element at ELEMENTPATH. For composite
            % properties or properties that reference other properties the
            % PARAMVALUE is the path to the corresponding property.
            %
            % Examples: 
            % Get the value of the IsService property for SenderReceiver
            % interface located at path 'Input'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % isService = dataObj.get('Input', 'IsService');
            %
            % See also set

            % Validate method arguments
            argParser = inputParser;   
            argParser.addRequired('ElementPath', @ischar);
            argParser.addRequired('PropertyName', @ischar);
            argParser.parse(elementPath, propertyName);   
            
            if strcmp(elementPath, 'XmlOptions')
                % XmlOptions are stored at AUTOSAR_ROOT
                elementPath = '/';
            end
            
            this.checkPath(elementPath);
            
            % Find the m3i object specified by elementPath
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            m3iObj   = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, elementPath) ;
            
            [~, ~, pathType] = autosar.api.getAUTOSARProperties.parseInputParams(varargin{:});
            
            % User can ask for Category of object
            if strcmp(propertyName, 'Category') && ~m3iObj.has('Category')
                metaClass = m3iObj.getMetaClass().name;
                paramValue = metaClass;
                return
            end
            
            % Check that object has the composite/non-composite property
            autosar.api.getAUTOSARProperties.validProperties(m3iObj, propertyName, 'mixed')
            
            paramValue = autosar.api.getAUTOSARProperties.getProperty(m3iObj, propertyName, pathType);
        end
    
        function set(this, elementPath, varargin)
            % SET(THIS, ELEMENTPATH, P1, V1) set the property P1 of AUTOSAR element located
            % at ELEMENTPATH to value V1. For properties that reference other
            % elements V1 should be a path. To set any of the XML options use
            % 'XmlOptions' as the ELEMENTPATH.
            %
            % Examples:
            % Set the IsService property for SenderReceiver interface located at
            % path 'Input' to true
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.set('Input', 'IsService', true);
            %
            % See also get
                
            % Validate method arguments
            argParser = inputParser;   
            argParser.addRequired('ElementPath', @ischar);
            argParser.parse(elementPath);            
            
            if strcmp(elementPath, 'XmlOptions')
                % XmlOptions are stored at AUTOSAR_ROOT
                elementPath = '/';
            end

            this.checkPath(elementPath);
            
            % Find the m3i object specified by path
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            m3iObj   = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, elementPath) ;

            % Which properties has user selected?
            [propertyNames, propertyValues] = autosar.api.getAUTOSARProperties.parseInputParams(varargin{:});

            % Check that object has all the non-composite properties
            autosar.api.getAUTOSARProperties.validProperties(m3iObj, propertyNames, 'noncomposite')
            
            % Now set properties
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before set');    
            trans = M3I.Transaction(m3iModel);
            autosar.api.getAUTOSARProperties.setProperties(m3iObj, propertyNames, propertyValues, this.ModelName);
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after set');    

            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
            
        function addPackageableElement(this, category, package, name, varargin)
            % ADDPACKAGEABLEELEMENT(THIS, CATEGORY, PACKAGE, NAME, P1, V1, ...)  
            % add an AUTOSAR element with category CATEGORY, package PACKAGE 
            % and name NAME. Additionally sets the property P1 to V1.
            %
            % Example:
            % Add a SenderReceiver interface with package '/A/B/C' and name
            % 'myInterface1' and IsService property set to true
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.addPackageableElement('SenderReceiverInterface',...
            %                   '/A/B/C', 'myInterface1', 'IsService', true);
            %             
            % Examples of other categories:
            %
            % dataObj.addPackageableElement('ModeSwitchInterface',...
            %            '/A/B/C', 'myInterface2');
            % dataObj.add('/A/B/C/myInterface2', 'ModeGroup', 'mgModes');
            %
            % dataObj.addPackageableElement('SwAddrMethod','/A/B/C/SwAddressMethods',...
            %    'sw1', 'MemoryAllocationKeywordPolicy', 'ADDR-METHOD-SHORT-NAME',...
            %    'SectionType', 'CODE')   
            %
            % dataObj.addPackageableElement('DataTypeMappingSet', '/A/B/C', 'myMappingSet1')
                        
            qname = [package '/' name];
            qname = strrep(qname, '//', '/');
            [nodePath, nodeName] = this.getOrAddPackage(qname);
            this.add( nodePath, 'packagedElement', nodeName,...
                'Category', category , varargin{:});
        end
 
        function add(this, parentPath, propertyName, name, varargin)
            % ADD(THIS, PARENTPATH, PROPERTY, NAME, P1, V1, ...) add a composite child
            % element with name NAME to AUTOSAR element at PARENTPATH under
            % property PROPERTY. The child will have property P1 set to V1.
            %
            % Example:
            % Add DataElement 'DE1' to SenderInterface 'Input'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.add('Input', 'DataElements', 'DE1');
            %
            % See also delete
            
            % Validate method arguments
            argParser = inputParser;   
            argParser.addRequired('ParentPath', @ischar);
            argParser.addRequired('PropertyName', @ischar);
            argParser.addRequired('Name', @ischar);
            argParser.parse(parentPath, propertyName, name);  
            
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);

            this.checkPath(parentPath);
            
            childPath = [parentPath, '/', name];
            
            % Check that child object does not already exist
            autosar.api.getAUTOSARProperties.checkNoObjByPartialOrFullPath(m3iModel, childPath);
            
            % Find parent object
            m3iParentObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, parentPath);
            
            % Check that object has the composite property
            autosar.api.getAUTOSARProperties.validProperties(m3iParentObj, propertyName, 'composite')            
            
            % Which properties has user selected?
            [childPropertyNames, childPropertyValues, ~, childCategory] = autosar.api.getAUTOSARProperties.parseInputParams(varargin{:});
            
            % What types can we use for the child object?
            if strcmp(propertyName, 'packagedElement')
                childMetaClass = Simulink.metamodel.foundation.PackageableElement.MetaClass;
            else
                childMetaClass = m3iParentObj.getMetaClass().getProperty(propertyName).type;
            end
            if isempty(childCategory)
                if childMetaClass.isAbstract
                    validCategoryNames = autosar.api.getAUTOSARProperties.concreteMetaClassNames(childMetaClass);
                    validCategoryNames = setdiff(validCategoryNames, this.UnsupportedCategories);
                    DAStudio.error('RTW:autosar:apiNeedCategory', childPath, ...
                        autosar.api.Utils.cell2str(validCategoryNames));
                end
            else
                % User must specify a category for heterogeneous composite properties
                validCategoryNames = autosar.api.getAUTOSARProperties.concreteMetaClassNames(childMetaClass);
                validCategoryNames = setdiff(validCategoryNames, this.UnsupportedCategories);
                invalidCategory = setdiff( childCategory, validCategoryNames );
                if ~isempty(invalidCategory)
                    validCategoryNames = sort(validCategoryNames);
                    DAStudio.error('RTW:autosar:apiInvalidCategory', invalidCategory{1}, childPath, ...
                        autosar.api.Utils.cell2str(validCategoryNames));
                end
                
                childMetaClass = autosar.api.getAUTOSARProperties.getMetaClassFromCategory(childCategory);              
            end
            childMetaClassQualifiedName = childMetaClass.qualifiedName;

            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before add');    
            trans = M3I.Transaction(m3iModel);
            m3iChildObj = feval(childMetaClassQualifiedName, m3iModel);

            % Add property to parent obj
            if this.isSequence(m3iParentObj.getMetaClass().getProperty(propertyName))
                m3iParentObj.(propertyName).append(m3iChildObj);
            else
                if m3iParentObj.(propertyName).isvalid()
                    % We don't want replace existing element, which would make the old element
                    % unparented
                    elemPath = [parentPath, '/', m3iParentObj.(propertyName).Name];
                    DAStudio.error('RTW:autosar:apiElementExists', elemPath);
                end
                m3iParentObj.(propertyName) = m3iChildObj;
            end

            % Check that object has all the non-composite properties
            autosar.api.getAUTOSARProperties.validProperties(m3iChildObj, childPropertyNames, 'noncomposite');
            
            % Assign name, value params
            autosar.api.getAUTOSARProperties.setProperty(m3iChildObj, 'Name', name, this.ModelName);
            % If a runnable is being created, then initialize the runnable
            % symbol with the value of runnable name
            if isa(m3iChildObj,...
                    'Simulink.metamodel.arplatform.behavior.Runnable')
                autosar.api.getAUTOSARProperties.setProperty(...
                    m3iChildObj,'symbol', name, this.ModelName);
            end
            autosar.api.getAUTOSARProperties.setProperties(m3iChildObj, childPropertyNames, childPropertyValues, this.ModelName);
           
            % Check that object would be valid by this transaction
            autosar.api.getAUTOSARProperties.isComplete(m3iChildObj);
            
            % Validate object
            validator = autosar.validation.MetaModelCommonValidator(this.ModelName);
            validator.verify(m3iChildObj);
            
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after add');    
            
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
        
        function delete(this, elementPath)
            % DELETE(THIS, ELEMENTPATH) delete the AUTOSAR element located at ELEMENTPATH.
            %
            % Example:
            % delete SenderReceiver interface 'Input'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.delete('Input');
            %
            % See also add

            % Validate method arguments
            argParser = inputParser;   
            argParser.addRequired('ElementPath', @ischar);
            argParser.parse(elementPath);       

            % error out for unnamed elements
            autosar.api.getAUTOSARProperties.checkIsNamedElement('delete', elementPath);
            
            this.checkPath(elementPath);
            
            % find object that we want to delete using path
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            m3iObj   = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, elementPath);
            
            % error checks for certain classes of objects when they are
            % referenced by other objects
            autosar.api.getAUTOSARProperties.checkObjectIsNotReferenced(m3iObj);
            
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before delete');    
            trans = M3I.Transaction(m3iModel);
            m3iObj.destroy();
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after delete');    
            
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end

        function deleteUnmappedComponents(this)
            % DELETEUNMAPPEDCOMPONENTS(THIS) delete all components other than the component
            % that is mapped to the Simulink model.
            %
            % Example:
            % delete all unmapped components
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.deleteUnmappedComponents();
            %
            % See also addPackageableElement

            mappedComponent = this.get('XmlOptions', 'ComponentQualifiedName');
            compositions = this.find([], 'CompositionComponent', 'PathType', 'FullyQualified');
            swcs = this.find([], 'AtomicComponent', 'PathType', 'FullyQualified');
            allComps = [compositions swcs];
            for ii=1:length(allComps)
                comp = allComps{ii};
                if ~strcmp(mappedComponent, comp)
                    % Not mapped component
                    this.delete(comp);
                end
            end
        end
        function createNumericType(this, name, varargin)
            %CREATENUMERICTYPE creates a Simulink.NumericType object. 
            %
            % Usage:
            %
            % createNumericType(this, NAME, COMPUMETHODPATH, IMPLEMENTATIONDATATYPEPATH)
            % creates Simulink.NumericType NAME from AUTOSAR CompuMethod
            % with Packageable path COMPUMETHODPATH and AUTOSAR
            % ImplementationDataType with Packageable path
            % IMPLEMENTATIONDATATYPEPATH. Simulink.NumericType is created
            % from CompuMethod with Linear and Identical Category.
            % NumericType scaling information comes from CompuMethod
            % Scaling. The DataTypeMode, Signedness and WordSize
            % information comes from ImplementationDataType.
            %
            % createNumericType(this, NAME, APPLICATIONDATATYPEPATH)
            % creates Simulink.NumericType NAME from AUTOSAR
            % ApplicationDataType with Packageable path
            % APPLICATIONDATATYPEPATH. Simulink.NumericType is created from
            % the CompuMethod referenced by ApplicationDataType, and the
            % ImplementationDataType mapped to this ApplicationDataType.
            %
            % Example:
            % Create a Simulink data type with a name myDataType from
            % CompuMethod with Packageable path '/a/b/myCM' and
            % implementationDataTypePath as
            % '/AUTOSAR_PlatformTypes/ImplementationDataTypes/uint32'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.createNumericType('myDataType', '/a/b/myCM', ...
            %   '/AUTOSAR_PlatformTypes/ImplementationDataTypes/uint32');
            
            narginchk(3, 4);
            if nargin == 3
                creatingFromAppType = true;
            else
                assert(nargin == 4, 'incorrect number of arguments passed to createNumericType.');
                creatingFromAppType = false;
            end
            
            isEnumType = false;
            [typeM3iObj, cmM3iObj] = this.parseArgsForCreateNumericOrEnumType(...
                name, creatingFromAppType, isEnumType, varargin{:});
            autosar.mm.mm2sl.TypeBuilder.createNumericTypeFromCompuMethod(...
                this.ModelName, name, cmM3iObj, typeM3iObj);
        end
        
        function createEnumeration(this, name, varargin)
            %CREATEENUMERATION creates Simulink Enumeration.
            %
            % Usage:
            %
            % createEnumeration(this, NAME, COMPUMETHODPATH, IMPLEMENTATIONDATATYPEPATH)
            % creates Simulink Enumeration NAME from AUTOSAR CompuMethod
            % with Packageable path COMPUMETHODPATH and AUTOSAR
            % ImplementationDataType with Packageable path
            % IMPLEMENTATIONDATATYPEPATH.
            %
            % createEnumeration(this, NAME, APPLICATIONDATATYPEPATH)
            % creates Simulink Enumeration from AUTOSAR ApplicationDataType
            % with Packageable path APPLICATIONDATATYPEPATH. Simulink
            % Enumeration is created from the CompuMethod referenced by
            % ApplicationDataType, and the ImplementationDataType mapped to
            % this ApplicationDataType.
            %
            % Example:
            % Create a Simulink enumeration with a name myEnum from AUTOSAR
            % CompuMethod with Packageable path '/a/b/myCM' and
            % implementationDataTypePath as
            % '/AUTOSAR_PlatformTypes/ImplementationDataTypes/uint16'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.createEnumeration('myEnum', '/a/b/myCM', ...
            %   '/AUTOSAR_PlatformTypes/ImplementationDataTypes/uint16');
            
            narginchk(3, 4);
            if nargin == 3
                creatingFromAppType = true;
            else
                assert(nargin == 4, 'incorrect number of arguments passed to createEnumeration.');
                creatingFromAppType = false;
            end
            
            isEnumType = true;
            [typeM3iObj, cmM3iObj] = this.parseArgsForCreateNumericOrEnumType(...
                name, creatingFromAppType, isEnumType, varargin{:});
            
            autosar.mm.mm2sl.TypeBuilder.createEnumerationFromCompuMethod(...
                name, cmM3iObj, typeM3iObj);
        end
        
        function createSystemConstants(this, elementPath)
            % CREATESYSTEMCONSTANTS(THIS, ELEMENTPATH) 
            % creates AUTOSAR.Parameter objects with CutomStorageClass
            % 'SystemConstant' from Packageable path ELEMENTPATH. The
            % ELEMENTPATH can be a PredefinedVariant Path or
            % cell array of SwSystemConstantValueSet Path. System Constant
            % objects are created with same name of SystemConstants. The
            % values for these objects will be evaluated from
            % SwSystemConstantValueSets.
            %
            % Example:
            % Create AUTOSAR.Parameter objects from all System Constants
            % referenced from PredefinedVariant '/Variants/Economy'.
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.createSystemConstants('/Variants/Economy');
            argParser = inputParser;
            argParser.addRequired('this', @(x) isa(x, class(x)));
            argParser.addRequired('ElementPath', @(x)(ischar(x) && ~isempty(x) || iscell(x)));
            argParser.parse(this, elementPath);
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            m3iObjs = {};
            m3iSystemConstantValueSets = {};
            if iscell(argParser.Results.ElementPath)
                for ii = 1 : numel(argParser.Results.ElementPath)
                    m3iObj = this.findObjByPartialOrFullPath(m3iModel, argParser.Results.ElementPath{ii});
                    m3iObjs = [m3iObjs m3iObj]; %#ok<AGROW>
                end
            else
                m3iObj = this.findObjByPartialOrFullPath(m3iModel, argParser.Results.ElementPath);
                m3iObjs = [m3iObjs m3iObj];
            end
            for ii = 1 : numel(m3iObjs)
                m3iObj = m3iObjs(ii);
                if isa(m3iObj, 'Simulink.metamodel.arplatform.variant.PredefinedVariant')
                    m3iSystemConstantValueSets = autosar.api.Utils.getSystemConstValueSetsFromPredefinedVariant(m3iObj, m3iSystemConstantValueSets);
                elseif isa(m3iObj, 'Simulink.metamodel.arplatform.variant.SystemConstValueSet')
                    m3iSystemConstantValueSets = [m3iSystemConstantValueSets m3iObj]; %#ok<AGROW>
                else
                    DAStudio.error('autosarstandard:api:invalidElementPath', elementPath, 'PredefinedVariant|SystemConstantValueSet', 'System Constants');
                end
            end
            if numel(m3iObjs) > 0
                autosar.api.Utils.createSystemConstantParams(this.ModelName, m3iSystemConstantValueSets);
            end
        end
    end
    
    methods(Static, Access=protected)
        % Subclasses may override these methods
        function attributeSeq = doFindCompositeAttributes(m3iObj)
            % Return sequence of composite attributes that can be added (using add method)
            
            isComposite = true;
            attributeSeq = Simulink.metamodel.arplatform.ModelFinder.findViewableAttributes(m3iObj, isComposite);
        end

        function attributeSeq = doFindNonCompositeAttributes(m3iObj)
            % Return sequence of primitive attributes that can get/set (using get/set methods)
            
            isComposite = false;
            attributeSeq = Simulink.metamodel.arplatform.ModelFinder.findViewableAttributes(m3iObj, isComposite);
        end
        
    end
    
    methods (Access=private)
        function [nodePath, nodeName] = getOrAddPackage(this, qualifiedName)
            % Helper method for:
            % 1) Doing some error checking on the qualified name
            % 2) Adding or retrieving the AR package for this qualified 
            %    name
            
            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qualifiedName, 'absPathShortName');            
            
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            
            [nodePath, nodeName] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qualifiedName);
            autosar.mm.Model.getOrAddARPackage(m3iModel, nodePath);
        end    
        
        function checkPath(this, pathStr)
            % Enforce paths are fully qualified when called in strict checking mode
            
            if this.IsStrict && ~strcmp(pathStr(1), '/')
                % Not fully qualified path
                DAStudio.error('autosarstandard:api:inefficientElementPath', pathStr);
            end
        end
        
        function [typeM3iObj, cmM3iObj] = parseArgsForCreateNumericOrEnumType(...
                this, name, creatingFromAppType, isEnumType, varargin)
            % parse arguments
            argParser = inputParser;
            argParser.addRequired('this', @(x) isa(x, class(x)));
            argParser.addRequired('Name', @(x)(ischar(x) && ~isempty(x)));
            if creatingFromAppType                
                argParser.addRequired('ApplicationDataTypePath', @(x)(ischar(x) && ~isempty(x)));
            else
                argParser.addRequired('CompuMethodPath', @(x)(ischar(x) && ~isempty(x)));
                argParser.addRequired('ImplementationDataTypePath', @(x)(ischar(x) && ~isempty(x)));
            end
            argParser.parse(this, name, varargin{:});
            
            % determine dataTypePath
            if (creatingFromAppType)
                dataTypePath = argParser.Results.ApplicationDataTypePath;
            else
                dataTypePath = argParser.Results.ImplementationDataTypePath;
            end
            
            if isEnumType
                slObjType = 'Simulink Enumeration';
            else
                slObjType = 'Simulink.NumericType';
            end
            
            % find dataType m3iObj
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            typeM3iObj = this.findObjByPartialOrFullPath(m3iModel, dataTypePath);
            if ~isa(typeM3iObj, 'Simulink.metamodel.types.PrimitiveType') ||...
                   creatingFromAppType && ~typeM3iObj.IsApplication 
                if creatingFromAppType
                    typeStr = 'ApplicationDataType';
                else
                    typeStr = 'ImplementationDataType';
                end
                DAStudio.error('autosarstandard:api:invalidElementPath', dataTypePath, typeStr, slObjType);
            end
            
            % find CompuMethod m3iObj
            if typeM3iObj.IsApplication
                cmM3iObj = typeM3iObj.CompuMethod;
                assert(~isempty(cmM3iObj), '%s does not reference a valid CompuMethod.', dataTypePath);
            else
                cmM3iObj = this.findObjByPartialOrFullPath(m3iModel, argParser.Results.CompuMethodPath);
                if ~isa(cmM3iObj, 'Simulink.metamodel.types.CompuMethod')
                    DAStudio.error('autosarstandard:api:invalidElementPath', compuMethodPath, 'CompuMethod', slObjType);
                end
            end
        end
    end
    
    methods(Static, Access=private)

        % --------------------------------------------------
        % Input parameter methods
        
        function [propertyNames, propertyValues, pathType, category, name] = parseInputParams(varargin)
            % Use inputParser to parse name value pairs
            p = inputParser;
            p.KeepUnmatched = true;
            p.PartialMatching = false;
            p.FunctionName = 'getAUTOSARProperties';
            hasCategory = false;
            if nargout > 3
                hasCategory = true;
                p.addParameter( 'Category', '' );
            end
            hasName = false;
            if nargout > 4
                hasName = true;
                p.addParameter( 'Name', '' );
            end
            
            p.addParameter( 'PathType', 'PartiallyQualified', @(x)any(validatestring(x,{'FullyQualified','PartiallyQualified'})));
            p.parse( varargin{:} );

            if hasCategory
                category = p.Results.Category;
            end
            if hasName
                name = p.Results.Name;
            end
            pathType = p.Results.PathType;
            
            params = p.Unmatched;
            propertyNames = fieldnames(params);
            propertyValues = struct2cell(params);
            
        end

        
        % --------------------------------------------------
        % Object property methods
        
        function setProperties(m3iObj, propertyNames, propertyValues, modelName)
            cmpResult = strcmp(propertyNames, 'MoveElements');
            if any(cmpResult)
                moveElementsMode = propertyValues{cmpResult};
            else
                moveElementsMode = 'All';
            end
            for ii=1:length(propertyNames)
                if strcmp('ComponentQualifiedName', propertyNames{ii})
                    % We need to keep in sync the component package with mapped component. Please
                    % see g888128
                    oldPropValue = autosar.api.getAUTOSARProperties.getProperty(m3iObj,...
                        propertyNames{ii}, 'FullyQualified');
                end
                
                autosar.api.getAUTOSARProperties.setProperty(m3iObj, propertyNames{ii}, propertyValues{ii}, modelName, moveElementsMode);

                if strcmp('ComponentQualifiedName', propertyNames{ii})
                    autosar.api.Utils.syncComponentQualifiedName(m3iObj,...
                        oldPropValue, propertyValues{ii});
                    % Update mapped component name in mapping as well
                     modelMapping = autosar.api.Utils.modelMapping(modelName);
                     if ~isempty(modelMapping.MappedTo)
                         % Transform the format
                         componentQName = strrep(propertyValues{ii}, '/', '.');
                         modelMapping.MappedTo.UUID = ['AUTOSAR' componentQName];
                     end
                end

            end
        end
        
        function setProperty(m3iObj, extPropertyName, propertyValue, modelName, moveElementsMode)

            import autosar.mm.util.XmlOptionsAdapter;
            import autosar.api.Utils;

            if nargin < 5
                moveElementsMode = 'All';
            end
            if isa(m3iObj, 'Simulink.metamodel.arplatform.common.AUTOSAR')
                if XmlOptionsAdapter.isProperty(extPropertyName)
                    XmlOptionsAdapter.verify(modelName, extPropertyName, propertyValue);
                    XmlOptionsAdapter.set(m3iObj, extPropertyName, propertyValue, moveElementsMode);
                    return;
                elseif strcmp(extPropertyName, 'DataTypePackage')
                    srcPkg = m3iObj.DataTypePackage;
                    errArgs = Utils.verifyXmlOptionsPackage(...
                        modelName, srcPkg, propertyValue, extPropertyName);
                    if isempty(errArgs{1})
                        Utils.moveImpDataTypes(m3iObj, srcPkg, propertyValue, moveElementsMode);
                    else
                         DAStudio.error(errArgs{:});
                    end
                elseif strcmp(extPropertyName, 'InterfacePackage')
                    intf = 'Simulink.metamodel.arplatform.interface';
                    srcPkg = m3iObj.InterfacePackage;
                    errArgs = Utils.verifyXmlOptionsPackage(...
                        modelName, srcPkg, propertyValue, extPropertyName);
                    if isempty(errArgs{1})
                        Utils.moveElementsByClassName(m3iObj,...
                            srcPkg, propertyValue, [intf, '.PortInterface'], moveElementsMode);
                    else
                        DAStudio.error(errArgs{:});
                    end
                end
            end
            
            m3iModel = autosar.api.Utils.m3iModel(modelName);
            if isa(m3iObj, 'Simulink.metamodel.arplatform.instance.TriggerInstanceRef') && ...
                strcmp(extPropertyName, 'Trigger')
                % There is a collision between the Event Trigger property
                % and Trigger instance reference Trigger property. This
                % ambiguity is solved here.
                propertyName = extPropertyName;
            else
                propertyName = autosar.api.getAUTOSARProperties.getIntPropertyName(extPropertyName);
            end
            if autosar.mm.util.ExternalToolInfoAdapter.isProperty(m3iObj, extPropertyName)
                autosar.mm.util.ExternalToolInfoAdapter.set(modelName, m3iObj, extPropertyName, propertyValue);
                return;
            end
            % local validation checks
            switch propertyName
              case 'Name'
                  autosar.api.Utils.checkQualifiedName(...
                      modelName, propertyValue, 'shortname');
                
                % Check for sibling name clashes
                m3iParentObj = m3iObj.containerM3I;
                siblingSeq = m3iParentObj.containeeM3I;
                for ii=1:siblingSeq.size()
                    m3iSiblingObj = siblingSeq.at(ii);
                    if m3iSiblingObj == m3iObj
                        continue;
                    end
                    
                    if isa(m3iSiblingObj, 'Simulink.metamodel.foundation.NamedElement')
                        if strcmp(propertyValue, m3iSiblingObj.Name)
                            siblingQName = autosar.api.Utils.getQualifiedName(m3iSiblingObj);
                            DAStudio.error('RTW:autosar:apiElementExists', siblingQName);
                        end
                    end
                end
              case 'ComponentQualifiedName'
                   autosar.api.Utils.checkQualifiedName(...
                      modelName, propertyValue, 'absPathShortName');                                        
              case 'symbol'
                  % Extra duplication check for runnable symbol
                  % Since symbol are used to generate the name of function, it
                  % cannot be the same in C scope.
                  if isa(m3iObj, 'Simulink.metamodel.arplatform.behavior.Runnable')
                      % Check length and if it's valid id
                      [isValid, errmsg] = autosar.validation.AutosarUtils.checkSymbol(propertyValue);
                      if ~isValid
                          error('RTW:autosar:invalidSymbol', ...
                              errmsg);
                      end
                      % Check for sibling name and symbol clash
                      hasClash = ...
                          autosar.api.Utils.checkRunnableSymbolClash(...
                          m3iObj, propertyValue);
                      if hasClash
                          DAStudio.error(...
                              'RTW:autosar:runnableSymbolClash',...
                              propertyValue);
                      end
                  end
                case 'DisplayFormat'
                    isAR4 = AR4.isCompatibleSchemaVer(get_param(modelName, 'AutosarSchemaVersion'));
                    if isAR4 && isa(m3iObj, 'Simulink.metamodel.types.CompuMethod') ||...
                            isa(m3iObj, 'Simulink.metamodel.arplatform.common.Data')
                        [isValid, ~] = autosar.validation.AutosarUtils.checkDisplayFormat(...
                            propertyValue, autosar.api.Utils.getQualifiedName(m3iObj));
                        if ~isValid
                            DAStudio.error('autosarstandard:ui:validateDisplayFormat', autosar.api.Utils.getQualifiedName(m3iObj));
                        end
                    end
                case 'Category'
                    if isa(m3iObj, 'Simulink.metamodel.types.CompuMethod')
                        if strcmp(propertyValue, 'RatFunc')
                            DAStudio.error('autosarstandard:common:CompuMethodCategoryRatFuncNotAllowed', m3iObj.Name)
                        elseif strcmp(propertyValue, 'LinearAndTextTable')
                            DAStudio.error('autosarstandard:common:CompuMethodCategoryLinearAndTextTableNotAllowed', m3iObj.Name)
                        end
                    end
                case 'Interface'
                    if isa(m3iObj, 'Simulink.metamodel.arplatform.port.ModeSenderPort')
                        msInterfaceObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, propertyValue);
                        errorIDAndHoles = autosar.validation.MetaModelValidator.verifyModeSwitchInterface(msInterfaceObj);
                        if ~isempty(errorIDAndHoles)
                            DAStudio.error(errorIDAndHoles{:});
                        end
                    end                
                otherwise
                % okay to fall through
            end    

            m3iProperty = m3iObj.getMetaClass().getProperty(propertyName);
            m3iPropertyType = m3iProperty.type;
            m3iPropertyTypeName = m3iPropertyType.name;             
            % For enumerations need to transform from string (e.g. 'Application') to
            % enumeration literal (e.g. 'Simulink.metamodel.arplatform.component.AtomicComponentKind.Application')
            if isa(m3iPropertyType, 'M3I.ImmutableEnumeration')
                % Check propertyValue is a valid enumeration literal
                validEnumerationLiterals = cell(1, m3iPropertyType.ownedLiteral.size());
                for ii=1:m3iPropertyType.ownedLiteral.size()
                    validEnumerationLiterals{ii} = m3iPropertyType.ownedLiteral.at(ii).name;
                end
                
                invalidEnumerationLiterals = setdiff( propertyValue, validEnumerationLiterals );
                if ~isempty(invalidEnumerationLiterals)
                    DAStudio.error('RTW:autosar:apiInvalidPropertyValue', propertyValue, propertyName, ...
                              autosar.api.Utils.cell2str(validEnumerationLiterals));
                end
                
                propertyValue = feval([m3iPropertyType.qualifiedName, '.', propertyValue]);
            end         
            
            if strcmp(m3iPropertyTypeName, 'FlowDataPortInstanceRef') || ...
                    strcmp(m3iPropertyTypeName, 'ModeDeclarationInstanceRef') || ...
                    strcmp(m3iPropertyTypeName, 'OperationPortInstanceRef') || ...
                    strcmp(m3iPropertyTypeName, 'TriggerInstanceRef')
                % Set Trigger port for DataReceivedEvent & ModeSwitchEvent, shielding user from
                % instanceRefs
                % 
                % Normally we would just assign a reference to the already
                % created instance reference object, however in this case we aid
                % the user by also creating the instanceRef

                isSeq = autosar.api.getAUTOSARProperties.isSequence(m3iProperty);
                propertyValues = cellstr(propertyValue);
                       
                % Create a fresh instance ref
                dataObj = autosar.api.getAUTOSARProperties(modelName);
                
                compQName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
                m3iCompSeq = autosar.mm.Model.findObjectByName(m3iModel, compQName);
                assert(m3iCompSeq.size() == 1)
                m3iComp = m3iCompSeq.at(1);

                instanceRefMetaClassQualifiedName = m3iPropertyType.qualifiedName;
                instanceRefMetaClassName = m3iPropertyTypeName;
                
                
                if isSeq
                    % clear the sequence before entering for-loop to append
                    % the new values
                    m3iObj.(propertyName).clear;
                end
                
                validTriggerIds = autosar.mm.util.InstanceRefAdapter.getValidShortIds(m3iComp, instanceRefMetaClassName);
                for ii=1:length(propertyValues)
                    propertyValue = propertyValues{ii};
                    
                    m3iInstanceRef = feval(instanceRefMetaClassQualifiedName, m3iModel);
                    if ~m3iComp.instanceMapping.isvalid()
                        m3iComp.instanceMapping = ...
                            Simulink.metamodel.arplatform.instance.ComponentInstanceRef(m3iModel);
                    end
                    m3iComp.instanceMapping.instance.append(m3iInstanceRef);

                    % Set properties of instance ref
                    instanceRefId = propertyValue;

                    if ~any(strcmp(instanceRefId, validTriggerIds))
                        % Invalid trigger id
                        DAStudio.error('RTW:autosar:apiInvalidPropertyValue', ...
                                       instanceRefId, ...
                                       extPropertyName, ...
                                       autosar.api.Utils.cell2str(validTriggerIds));
                    end
                    
                    % Expect 'PortName.ElementName'
                    dotIdx = find(instanceRefId == '.');
                    portName = instanceRefId(1:dotIdx-1);
                    arg2 = instanceRefId(dotIdx+1:end);
                    
                    portQName = [compQName, '/', portName];
                    interfaceQName = dataObj.get(portQName, 'Interface', 'PathType', 'FullyQualified');               
                    
                    autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'Port', portQName, modelName);
                    if strcmp(m3iPropertyTypeName, 'FlowDataPortInstanceRef')
                        elementName = arg2;
                        elementQName = [interfaceQName, '/', elementName]; 
                        autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'DataElements', elementQName, modelName);
                    elseif strcmp(m3iPropertyTypeName, 'ModeDeclarationInstanceRef')
                        modeDeclGroupElementQName = dataObj.get(interfaceQName, 'ModeGroup', 'PathType', 'FullyQualified');
                        m3iModeDeclGroupElement = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, modeDeclGroupElementQName);
                        m3iModeGroup = m3iModeDeclGroupElement.ModeGroup;
                        modeGroupQName = autosar.api.Utils.getQualifiedName(m3iModeGroup);
                        
                        modeName = arg2;
                        modeQName = [modeGroupQName, '/', modeName];
                        
                        autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'Mode', modeQName, modelName);
                        autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'groupElement', modeDeclGroupElementQName, modelName);
                    elseif strcmp(m3iPropertyTypeName, 'TriggerInstanceRef')
                        elementName = arg2;
                        elementQName = [interfaceQName, '/', elementName]; 
                        autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'Trigger', elementQName, modelName);                        
                    else
                        elementName = arg2;
                        elementQName = [interfaceQName, '/', elementName]; 
                        autosar.api.getAUTOSARProperties.setProperty(m3iInstanceRef, 'Operations', elementQName, modelName);
                    end
                    
                    % Assign reference
                    if isSeq
                        m3iObj.(propertyName).append(m3iInstanceRef);
                    else
                        m3iObj.(propertyName) = m3iInstanceRef;
                    end
                end
            elseif autosar.api.getAUTOSARProperties.isReference(m3iProperty)
                % propertyValue is the name of the referenced object
                m3iModel = m3iObj.rootModel;
                isSeq = autosar.api.getAUTOSARProperties.isSequence(m3iProperty);
                propertyValues = cellstr(propertyValue);
                
                if isSeq
                    % clear the sequence before entering for-loop to append
                    % the new values
                    m3iObj.(propertyName).clear;
                else
                    assert(isscalar(propertyValues),...
                        'The property value for "%s" should be a scalar.', propertyName);
                end
                
                for ii=1:numel(propertyValues)
                    propertyValue = propertyValues{ii};
                    m3iRefObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath( m3iModel, propertyValue );
                    % Add property to parent obj
                    if isSeq                       
                        m3iObj.(propertyName).append(m3iRefObj);
                    else
                        m3iObj.(propertyName) = m3iRefObj;
                    end
                end                
            else
                % type cast from MATLAB to M3I
                m3iObj.(propertyName) = propertyValue;
            end
                        
            % When adding checks for new property, add your check inside
            % MetaModelCommonValidator. Verify the properties after setting
            % them on the meta-model but before commit (M3i.Transcation not
            % committed yet)
            metaModelCommonValidator = autosar.validation.MetaModelCommonValidator(modelName);            
            switch propertyName
                case 'LiteralPrefix'
                    metaModelCommonValidator.verifyProperties(m3iObj, {'LiteralPrefix'});
                otherwise
                    % okay to fall through
            end
        end
    
        
        function propertyValue = getProperty(m3iObj, extPropertyName, pathType)
            
            import autosar.mm.util.XmlOptionsAdapter;
            % Check external tool info for the property first.
            if isa(m3iObj, 'Simulink.metamodel.arplatform.common.AUTOSAR') && ...
                    XmlOptionsAdapter.isProperty(extPropertyName)
                
                propertyValue = XmlOptionsAdapter.get(m3iObj, ...
                                                      extPropertyName);
                return;
            end
            
            propertyName = autosar.api.getAUTOSARProperties.getIntPropertyName(extPropertyName);
            % M3I.ImmutableProperty
            m3iProperty = m3iObj.getMetaClass().getProperty(propertyName);
            if isempty(m3iProperty) && autosar.mm.util.ExternalToolInfoAdapter.isProperty(m3iObj, extPropertyName)
                propertyValue = autosar.mm.util.ExternalToolInfoAdapter.get(m3iObj, extPropertyName);
                return;
            end
            m3iPropertyType = m3iProperty.type;
            m3iPropertyTypeName = m3iPropertyType.name; 
            isSeq = autosar.api.getAUTOSARProperties.isSequence(m3iProperty);
                        
            % For enumerations need to transform from enumeration literal to string
            if isa(m3iPropertyType, 'M3I.ImmutableEnumeration')
                
                if isSeq
                    seqOfValues = m3iObj.get(propertyName);
                    propertyValue = m3i.mapcell(@toString, seqOfValues);
                else
                    propertyValue = m3iObj.getOne(propertyName).toString();
                end

            elseif strcmp(m3iPropertyTypeName, 'FlowDataPortInstanceRef')
                m3iChildObj = m3iObj.(propertyName);
                if m3iChildObj.isvalid()
                    propertyValue = [m3iChildObj.Port.Name, '.' m3iChildObj.DataElements.Name];
                else
                    propertyValue = [];
                end
            elseif strcmp(m3iPropertyTypeName, 'OperationPortInstanceRef')
                m3iChildObj = m3iObj.(propertyName);
                if m3iChildObj.isvalid()
                    propertyValue = [m3iChildObj.Port.Name, '.' m3iChildObj.Operations.Name];
                else
                    propertyValue = [];
                end
            elseif strcmp(m3iPropertyTypeName, 'ModeDeclarationInstanceRef')
                m3iChildObj = m3iObj.(propertyName);
                if m3iChildObj.isvalid()
                    propertyValue = cell(1, m3iChildObj.size());
                    for ii=1:m3iChildObj.size()
                        propertyValue{ii} = [m3iChildObj.at(ii).Port.Name, '.', m3iChildObj.at(ii).Mode.Name];
                    end
                else
                    propertyValue = [];
                end
            elseif autosar.api.getAUTOSARProperties.isReference(m3iProperty)
                
                % check if referenced object actually exists
                m3iChildObj = m3iObj.(propertyName);
                if ~m3iChildObj.isvalid()
                    propertyValue = [];
                    return
                end 
                
                switch pathType
                  case 'FullyQualified'
                    fh = @autosar.api.Utils.getQualifiedName;
                  case 'PartiallyQualified'
                    fh = @autosar.api.getAUTOSARProperties.getPartiallyQualifiedName;
                  otherwise
                    assert(false, 'Did not recognize pathType %s', pathType);
                end
                
                % propertyValue is the name of the referenced object
                if isSeq
                    propertyValue = m3i.mapcell(fh, m3iChildObj);
                else
                    propertyValue = fh(m3iChildObj);           
                end

            else
                m3iChildObj = m3iObj.(propertyName);
                
                % type cast from M3I to MATLAB
                if isSeq
                    propertyValue = m3i.mapcell(@(x) x, m3iChildObj);
                    
                    % if numeric convert to matrix
                    if length(propertyValue) >= 1
                        if isnumeric(propertyValue{1})
                            propertyValue = cell2mat(propertyValue);
                        end
                    end
                    
                else
                    propertyValue = m3iChildObj;
                end
                
            end
            
        end
        
        
        function validProperties(m3iObj, propertyNames, compositeType)
        % Check that object has valid attributes

            propertyNames = cellstr(propertyNames);

            % Only run further property checks if we need to
            runChecks = false;
            for ii=1:length(propertyNames)
                propertyName = propertyNames{ii};

                if ~m3iObj.has(propertyName)
                    % Invalid property
                    runChecks = true;
                    break
                end
                
                m3iProperty = m3iObj.getMetaClass().getProperty(propertyName);
                switch compositeType
                  case 'composite'
                    isViewable = Simulink.metamodel.arplatform.ModelFinder.isViewableAttribute(m3iProperty, true);
                  case 'noncomposite'
                    isViewable = Simulink.metamodel.arplatform.ModelFinder.isViewableAttribute(m3iProperty, false);
                  case 'mixed'
                    isViewable = Simulink.metamodel.arplatform.ModelFinder.isViewableAttribute(m3iProperty, true) || ...
                        Simulink.metamodel.arplatform.ModelFinder.isViewableAttribute(m3iProperty, false);
                  otherwise
                    assert(false, 'Did not recognize compositeType %s', compositeType);
                end     
                    
                if ~isViewable
                    % Property not viewable
                    runChecks = true;
                    break
                end
            end

            if ~runChecks
                return
            end
            
            % Need to error out
            switch compositeType
              case 'composite'
                validAttributesSeq = autosar.api.getAUTOSARProperties.doFindCompositeAttributes(m3iObj);
              case 'noncomposite'
                validAttributesSeq = autosar.api.getAUTOSARProperties.doFindNonCompositeAttributes(m3iObj);                
              case 'mixed'
                validAttributesSeq = autosar.api.getAUTOSARProperties.doFindCompositeAttributes(m3iObj);
                validNonCompositeAttributesSeq = autosar.api.getAUTOSARProperties.doFindNonCompositeAttributes(m3iObj);                
                
                validAttributesSeq.addAll(validNonCompositeAttributesSeq);
              otherwise
                assert(false, 'Did not recognize compositeType %s', compositeType);
            end
               
           validAttributes = cell(1, validAttributesSeq.size());
           for ii = 1:validAttributesSeq.size()
               validAttributes{ii} = autosar.api.getAUTOSARProperties.getExtPropertyName(validAttributesSeq.at(ii).name);
           end
           
           if isa(m3iObj, 'Simulink.metamodel.arplatform.common.AUTOSAR')
               xmlProps = autosar.mm.util.XmlOptionsAdapter.getValidProperties();
               validAttributes = [validAttributes, xmlProps{:}];
           end
           externalToolInfoProps = autosar.mm.util.ExternalToolInfoAdapter.getValidProperties(m3iObj);
           validAttributes = [validAttributes, externalToolInfoProps{:}];

           invalidAttributes = setdiff( propertyNames, validAttributes );
           
           % The IRV class inherits the InvalidationPolicy attribute from
           % FlowData. This attribute should not be inherited and we are
           % eliminating it from the list displayed in the error message.
           % This logic can be eliminated when the metamodel is fixed.
           if isa(m3iObj, 'Simulink.metamodel.arplatform.behavior.IrvData')
               validAttributes = setdiff(validAttributes, 'InvalidationPolicy');
           end

           % DataElements of NvDataInterfaces don't have InvalidationPolicy
           if isa(m3iObj.containerM3I, 'Simulink.metamodel.arplatform.interface.NvDataInterface') && ...
              isa(m3iObj, 'Simulink.metamodel.arplatform.interface.FlowData')
               validAttributes = setdiff(validAttributes, 'InvalidationPolicy');
           end
               
           % need to accept some hidden attributes
           validHiddenAttributes = {'packagedElement'};
           invalidAttributes = setdiff( invalidAttributes, validHiddenAttributes );
           if ~isempty(invalidAttributes)
               invalidAttributes = setdiff(invalidAttributes, validHiddenAttributes);
           end
           if ~isempty(invalidAttributes)
               elementName = autosar.api.Utils.getQualifiedName(m3iObj);
               DAStudio.error('RTW:autosar:apiInvalidProperties', elementName, autosar.api.Utils.cell2str(invalidAttributes), ...
                              autosar.api.Utils.cell2str(validAttributes));
               
           end
        end

        function intPropertyName = getIntPropertyName(extPropertyName)

            switch extPropertyName
              case 'Trigger'
                intPropertyName = 'instanceRef';
              case 'Activation'
                intPropertyName = 'activation';    
              otherwise
                intPropertyName = extPropertyName;
            end
            
        end

        
        function extPropertyName = getExtPropertyName(intPropertyName)
                    
            switch intPropertyName
              case 'instanceRef'
                extPropertyName = 'Trigger';
              case 'activation'
                extPropertyName = 'Activation';
              otherwise
                extPropertyName = intPropertyName;
            end
            
        end

        
        function metaClassNames = concreteMetaClassNames( metaClass )
            % find concrete meta class names recursing through subClasses for abstract classes

            metaClassNames = {};
            if metaClass.isAbstract
                % recurse through subClasses
                subClassesSeq = metaClass.subClass;
                for ii=1:subClassesSeq.size()
                    subClass = subClassesSeq.at(ii);
                    if subClass.has('extension_m3i_hide_in_mcos')
                        % ignore hidden types
                    else
                        metaClassNames = [metaClassNames, ...
                                          autosar.api.getAUTOSARProperties.concreteMetaClassNames(subClass)]; %#ok<AGROW>
                    end
               end
            else
               metaClassNames = {metaClass.name};
            end
        end        

        
        function metaClassNames = abstractMetaClassNames( metaClass )
            % find abstract meta class names recursing through subClasses for abstract classes

            metaClassNames = {};
            if metaClass.isAbstract
                metaClassNames = {metaClass.name};
                % recurse through subClasses
                subClassesSeq = metaClass.subClass;
                for ii=1:subClassesSeq.size()
                    subClass = subClassesSeq.at(ii);
                    if subClass.has('extension_m3i_hide_in_mcos')
                        % ignore hidden types
                    else
                        metaClassNames = [metaClassNames, ...
                                          autosar.api.getAUTOSARProperties.abstractMetaClassNames(subClass)]; %#ok<AGROW>
                    end
               end
            end
        end        

        function checkObjectIsNotReferenced(m3iObj)
            % Don't delete an interface that is still referenced by one or more ports
            if isa(m3iObj, 'Simulink.metamodel.arplatform.interface.PortInterface') && ~m3iObj.Ports.isEmpty()
                DAStudio.error('RTW:autosar:apiDeleteReferencedElementErr', 'interface', m3iObj.Name, 'ports', 'interface');
            end
            
            % Don't allow deleting a component which is referenced by a
            % composition
            m3iCompositions = autosar.api.Utils.findCompositionsUsingComponent(m3iObj);
            if ~isempty(m3iCompositions)
                DAStudio.error('RTW:autosar:apiDeleteReferencedElementErr', 'component', m3iObj.Name, 'compositions', 'component');
            end
        end
        
        function checkIsNamedElement(methodName, elementName)
            % Error out for unnamed elements
            if ~autosar.api.Utils.isNamedElement(elementName)
                DAStudio.error('RTW:autosar:apiUnnamedElementNotSupported', methodName, elementName);
            end
        end
        
        function checkNoObjByPartialOrFullPath( m3iModel, elementPath )
           % Error out if object exists
            
            m3iObjSeq = autosar.api.getAUTOSARProperties.findObjSeqByPartialOrFullPath( m3iModel, elementPath );
            if m3iObjSeq.size() ~= 0
                DAStudio.error('RTW:autosar:apiElementExists', elementPath);
            end
        
        end

        function m3iObj = findObjByPartialOrFullPath( m3iModel, elementPath )
            if autosar.api.Utils.isNamedElement(elementPath)
                m3iObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPathForNamedElement( m3iModel, elementPath );
            else
                m3iObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPathForUnnamedElement( m3iModel, elementPath );
            end
        end
        
        function m3iObj = findObjByPartialOrFullPathForNamedElement( m3iModel, elementPath )
            m3iObjSeq = autosar.api.getAUTOSARProperties.findObjSeqByPartialOrFullPath( m3iModel, elementPath );
            switch m3iObjSeq.size()
                case 0
                    DAStudio.error('RTW:autosar:apiInvalidPath', elementPath);
                case 1
                    m3iObj = m3iObjSeq.at(1);
                otherwise
                    qnames = cell(1, m3iObjSeq.size);
                    for ii=1:m3iObjSeq.size()
                        qnames{ii} = autosar.api.Utils.getQualifiedName(m3iObjSeq.at(ii));
                    end
                    
                    DAStudio.error('RTW:autosar:apiNeedFullyQualifiedPath', elementPath, ...
                        autosar.api.Utils.cell2str(qnames));
            end
        end
        
        function m3iObj = findObjByPartialOrFullPathForUnnamedElement( m3iModel, elementPath )
            % Keep walking backwards in the unnamed element path until we
            % find a named parent. Collect the property names and indices
            % by decoding the unnamed elements in the path.
            propNames = [];
            elementIndices = [];
            parentPath = elementPath;
            while (~autosar.api.Utils.isNamedElement(parentPath))
                [parentPath, childName] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(parentPath);
                [propNames{end+1}, elementIndices{end+1}] = autosar.api.UnnamedElement.decodeName(childName); %#ok<AGROW>
            end
            
            % using the property names and indices collected above, now
            % navigate to the m3iObj we are interested in
            m3iObj = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPathForNamedElement( m3iModel, parentPath );
            for unnamedIdx = length(propNames):-1:1
                propertyName = propNames{unnamedIdx};
                elementIndex = elementIndices{unnamedIdx};
                if (elementIndex ~= -1)
                    % in a sequence
                    m3iSeq = m3iObj.(propertyName);
                    assert(elementIndex > 0 && elementIndex <= m3iSeq.size(),...
                        'Invalid access as index %d is out of bounds on %s, which is of size %d.',...
                        elementIndex, propertyName, m3iSeq.size());
                    m3iObj = m3iSeq.at(elementIndex);
                else
                    m3iObj = m3iObj.(propertyName);
                end
            end
        end
           
        function m3iObjSeq = findObjSeqByPartialOrFullPath( m3iModel, elementPath )
            % PATH can be fully qualified name, for example '/A/B/C/swcComponent/RPort1' or
            % a path rooted to either the component or interface, for example 'swcComponent/RPort1'.
            
            if length(elementPath) == 1 && strcmp(elementPath(1), '/')
                % return root package
                assert(m3iModel.RootPackage.size() == 1);
                m3iObjSeq = M3I.SequenceOfClassObject.make(m3iModel.rootModel);
                m3iObjSeq.append(m3iModel.RootPackage.front());
            elseif length(elementPath) > 1 && strcmp(elementPath(1), '/')
                % elementPath is a fully qualified name
                m3iObjSeq = autosar.mm.Model.findObjectByName( m3iModel, elementPath );
            else
                % elementPath is a partially qualified name
                m3iObjSeq = autosar.api.getAUTOSARProperties.findPackagedObjSeqByName( m3iModel, elementPath );
            end

        end
        
        function m3iPackagedObjSeq = findPackagedObjSeqByName( m3iModel, name )
            m3iPackagedObjSeq = M3I.SequenceOfClassObject.make(m3iModel.rootModel);

            % Find packages
            m3iPkgSeq = Simulink.metamodel.arplatform.ModelFinder.findObjectByMetaClass(m3iModel, Simulink.metamodel.arplatform.common.Package.MetaClass, true);
            
            % Loop through packages looking for object
            for ii=1:m3iPkgSeq.size()
                m3iObjSeq = autosar.mm.Model.findObjectByName(m3iPkgSeq.at(ii), name);
                
                for jj=1:m3iObjSeq.size()
                    m3iPackagedObjSeq.append(m3iObjSeq.at(jj));
                end
            end
        end
        
        function isSeq = isSequence(m3iProperty)
            isSeq = ~strcmp(m3iProperty.upper, '1');            
        end
        
        function isRef = isReference(m3iProperty)
            isRef = ~isempty(m3iProperty.association);
        end
        
        function path = getPartiallyQualifiedName(m3iObj)
            qualifiedName = autosar.api.Utils.getQualifiedName(m3iObj);
            
            m3iPkgObj = m3iObj.containerM3I;
            if ~isa(m3iPkgObj, 'Simulink.metamodel.arplatform.common.AUTOSAR')
                while ~isa(m3iPkgObj, 'Simulink.metamodel.arplatform.common.Package') 
                    m3iPkgObj = m3iPkgObj.containerM3I;
                end
            end
            
            packageQualifiedName = autosar.api.Utils.getQualifiedName(m3iPkgObj);
            path = regexprep( qualifiedName, ['^', packageQualifiedName, '/'], '');
        end
        
        function isComplete(m3iObj)
           % Check atomicity of object
           
           if isa(m3iObj, 'Simulink.metamodel.arplatform.port.Port')
              % Ports must reference a Type (redefined as Interface for 
              % the ports that we're interested in)
              if ~m3iObj.Type.isvalid()
                DAStudio.error('RTW:autosar:apiInvalidObj', 'Interface', ...
                    autosar.api.Utils.getQualifiedName(m3iObj));
              end
           end       
            
        end
    
        % Helper method to set the Symbol and BaseExponent(s) for Dimension
        function setPhysicalDimensionAttributes(m3iDim, exponents)            
            m3iDim.BaseExponent.clear();
            symbol = '';
            if (exponents.LengthExp ~= 0)
                m3iDim.BaseExponent.append(exponents.LengthExp);
                symbol = [symbol 'l'];
            end
            if (exponents.MassExp ~= 0)
                m3iDim.BaseExponent.append(exponents.MassExp);
                symbol = [symbol 'm'];
            end
            if (exponents.TimeExp ~= 0)
                m3iDim.BaseExponent.append(exponents.TimeExp);
                symbol = [symbol 't'];
            end
            if (exponents.CurrentExp ~= 0)
                m3iDim.BaseExponent.append(exponents.CurrentExp);
                symbol = [symbol 'I'];
            end
            if (exponents.TemperatureExp ~= 0)
                m3iDim.BaseExponent.append(exponents.TemperatureExp);
                symbol = [symbol 'T'];
            end
            if (exponents.MolarAmountExp ~= 0)
                m3iDim.BaseExponent.append(exponents.MolarAmountExp);
                symbol = [symbol 'n'];
            end
            if (exponents.LuminousIntensityExp ~= 0)
                m3iDim.BaseExponent.append(exponents.LuminousIntensityExp);
                symbol = [symbol 'L'];
            end
            m3iDim.Symbol = symbol;
        end
    end
    
    methods(Hidden, Access = public)
        % These add methods are hidden since they are now replaced with the
        % addPackageableElement API. However, we cannot remove them since we
        % exposed them for at least one release and don't want cause any 
        % backwards incompatibility issues. At some point, we can deprecate
        % these APIs.        
        function addSRInterface(this, qname, varargin)
            % ADDSRINTERFACE(THIS, QNAME, P1, V1, ...) add a Sender-Receiver interface with
            % qualified name QNAME. Additionally sets the property P1 to V1.
            %
            % Example:
            % Add a SenderReceiver interface with qualified name
            % '/A/B/C/myInterface1' and IsService property set to true
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.addSRInterface('/A/B/C/myInterface1', 'IsService', true);
            
            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qname, 'absPathShortName');
            [package,name] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);
            this.addPackageableElement('SenderReceiverInterface', package, name, varargin{:});
        end
        
        function addMSInterface(this, qname, varargin)
            % ADDMSINTERFACE(THIS, QNAME, P1, V1, ...) add a Mode-Switch interface with
            % qualified name QNAME. Additionally sets the property P1 to V1.
            %
            % Example:
            % Add a ModeReceiver interface with qualified name
            % '/A/B/C/myInterface1' and composite attribute ModeGroup set
            % to 'mgModes'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.addMSInterface('/A/B/C/myInterface1', 'IsService', true);
            % dataObj.add('/A/B/C/myInterface1', 'ModeGroup', 'mgModes');
            
            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qname, 'absPathShortName');
            [package,name] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);
            this.addPackageableElement('ModeSwitchInterface', package, name, varargin{:});                        
        end
 
        function addSwAddrMethod(this, qname, varargin)
            % ADDSWADDRMETHOD(THIS, QNAME, P1, V1, ...) add a SwAddrMethod with
            % qualified name QNAME. Additionally sets the property P1 to V1.
            %
            % Example:
            % Add a SwAddrMethod with qualified name
            % '/A/B/C/myAddrMethod' and MemoryAllocationKeywordPolicy
            % property set to 'ADDR-METHOD-SHORT-NAME'
            % and SectionType property set to 'VAR'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.addSwAddrMethod('/A/B/C/SwAddressMethods/sw1',...
            %    'MemoryAllocationKeywordPolicy', 'ADDR-METHOD-SHORT-NAME',...
            %    'SectionType', 'CODE')          

            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qname, 'absPathShortName');
            [package,name] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);
            this.addPackageableElement('SwAddrMethod', package, name, varargin{:});
        end
        
        function addComponent(this, qname, varargin)

            % Error out if model is already mapped to a component
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            if ~isempty(modelMapping.MappedTo)
                DAStudio.error('RTW:autosar:apiMappingAlreadyHasComponent',modelMapping.MappedTo.Name);
            end

            [compPath, compName] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);

            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            autosar.mm.Model.getOrAddARPackage(m3iModel, compPath);

            this.add( compPath, 'packagedElement', compName, 'Category', 'AtomicComponent', varargin{:});
            
            % Add behavior
            this.add( qname, 'Behavior', 'Behavior' );
            
            % Change ComponentQualifiedName
            this.set('XmlOptions', 'ComponentQualifiedName', qname);
            
            % Mapping is the last thing we do thus ensuring this function is exception safe
            m3iSWC = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath( m3iModel, qname );
            compObj = Simulink.AutosarTarget.Component(m3iSWC.qualifiedName, m3iSWC.Name); 
            modelMapping.mapComponent(compObj);
        end

        function moveElements(this, srcPkg, destPkg, category)
            % MOVEELEMENTS(THIS, SRCPKG, DESTPKG, CATEGORY) move
            % Packageable elements from source Package to destination
            % Package. Destination Package will be created if it does not
            % exist.
            %
            % Example:
            % Add all CompuMethods in Package '/a/b/CompuMethods' to Package '/x/y/z/CompuMethods'
            %
            % dataObj = ...
            % autosar.api.getAUTOSARProperties('rtwdemo_autosar_counter');
            % dataObj.moveElements('/a/b/CompuMethods', '/x/y/z/CompuMethods', 'CompuMethods');
            %
            
            % Validate method arguments            
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            m3iSrcPkg = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, srcPkg);
            childMetaClass = m3iSrcPkg.getMetaClass().getProperty('packagedElement').type;
            validCategoryNames = autosar.api.getAUTOSARProperties.concreteMetaClassNames(childMetaClass);
            invalidCategory = setdiff( category, validCategoryNames );
            if ~isempty(invalidCategory)
                DAStudio.error('RTW:autosar:apiInvalidCategory', invalidCategory{1}, childPath, ...
                    autosar.api.Utils.cell2str(validCategoryNames));
            end
            
            childMetaClass = autosar.api.getAUTOSARProperties.getMetaClassFromCategory(category);
            metaClsStr = childMetaClass.qualifiedName;
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before moveElements');    
            trans = M3I.Transaction(m3iModel);
            autosar.api.Utils.moveElementsByClassName(m3iModel, srcPkg, destPkg,...
                metaClsStr, 'All')
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after moveElements');    
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
                                
        function addUnit(this, qname, varargin)
            % ADDUNIT(THIS, QNAME, P1, V1, ...) add a Unit with
            % qualified name QNAME. Additionally sets the property P1 to V1.
            %
            % Example:
            % dataObj = autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.addUnit('/units/unit1',...
            %                 'DisplayName', 'u1',...
            %                 'FactorSiToUnit', 1, 'OffsetSiToUnit', -2,...
            %                 'PhysicalDimension', '/dims/Len1');
            
            % check qualified name
            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qname, 'absPathShortName');
            
            % parse arguments
            argParser = inputParser;
            argParser.addParameter('DisplayName', '', @(x)(ischar(x)));
            argParser.addParameter('FactorSiToUnit', 1, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('OffsetSiToUnit', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('PhysicalDimension', '', @(x)(ischar(x)));
            argParser.parse(varargin{:});
            
            % get dimension object
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            dimQName = argParser.Results.PhysicalDimension;
            m3iDim = [];
            if ~isempty(dimQName)
                m3iDim = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel,...
                    dimQName);
                assert(~isempty(m3iDim), '%s element does not exist.', dimQName);
            end
            
            % Add the unit
            [parentPath, childName] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before addUnit');    
            trans = M3I.Transaction(m3iModel);
            this.addPackageableElement('Unit', parentPath, childName);
            
            % set unit attributes. These cannot be passed into
            % addPackageableElement because they are not public attributes
            m3iUnit = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, qname);
            m3iUnit.Symbol = argParser.Results.DisplayName;
            m3iUnit.ConvFactor = argParser.Results.FactorSiToUnit;
            m3iUnit.ConvOffset = argParser.Results.OffsetSiToUnit;
            if ~isempty(m3iDim)
                m3iUnit.Dimension = m3iDim;
            end
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after addUnit');    
        end
        
        function addPhysicalDimension(this, qname, varargin)
            % ADDPHYSICALDIMENSION(THIS, QNAME, P1, V1, ...) add a 
            % PhysicalDimension with qualified name QNAME. Additionally 
            % sets the property P1 to V1.
            %
            % Example:            
            % dataObj = autosar.api.getAUTOSARProperties(mdlName);
            % dataObj.addPhysicalDimension('/dims/Len1M2Ti3I4T5Mol6CdNeg7',...
            %                              'LengthExp', 1,...
            %                              'MassExp', 2,...
            %                              'TimeExp', 3,...
            %                              'CurrentExp', 4,...
            %                              'TemperatureExp', 5,...
            %                              'MolarAmountExp', 6,...
            %                              'LuminousIntensityExp', -7);
                        
            % check qualified name
            autosar.api.Utils.checkQualifiedName(...
                this.ModelName, qname, 'absPathShortName');
            
            % parse arguments
            argParser = inputParser;
            argParser.addParameter('LengthExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('MassExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('TimeExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('CurrentExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('TemperatureExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('MolarAmountExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.addParameter('LuminousIntensityExp', 0, @(x)(~isempty(x) && isfloat(x)));
            argParser.parse(varargin{:});
            
            % add the element
            m3iModel = autosar.api.Utils.m3iModel(this.ModelName);
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects before addPhysicalDimension');    
            trans = M3I.Transaction(m3iModel);
            [parentPath, childName] = autosar.mm.sl2mm.ModelBuilder.getNodePathAndName(qname);
            this.addPackageableElement('Dimension', parentPath, childName);
            
            % set exponents and symbol. These cannot be passed into
            % addPackageableElement because they are not public attributes
            m3iDim = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, qname);
            autosar.api.getAUTOSARProperties.setPhysicalDimensionAttributes(...
                m3iDim, argParser.Results);
            trans.commit();
            assert(m3iModel.unparented.isEmpty(), 'Unparented objects after addPhysicalDimension');    
        end
    end
    methods(Static, Hidden, Access = public)
        function paths = find_impl(m3iModel, rootPath, category, varargin)
            persistent componentAttributes;
            persistent ibAttributes;

            argParser = inputParser;   
            argParser.addRequired('RootPath', @(x)(ischar(x)||isempty(x)));
            argParser.addRequired('Category', @ischar);
            argParser.parse(rootPath, category);   
            
            metaClass = autosar.api.getAUTOSARProperties.getMetaClassFromCategory(category);            
            [propertyNames, propertyValues, pathType, categoryProp, objName] = autosar.api.getAUTOSARProperties.parseInputParams(varargin{:});
            if ~isempty(categoryProp)
                propertyNames{end+1} = 'Category';
                propertyValues{end+1} = categoryProp;
            end
            recursiveSearch = true;
            searchBySuperClass = true;
            if ~isempty(rootPath) && ~strcmp(rootPath, '/')
                m3iRootObj   = autosar.api.getAUTOSARProperties.findObjByPartialOrFullPath(m3iModel, rootPath);
                % Following code is added to improve the Performance. For
                % example the Ports always found under Component and there
                % is no need to do recursive search. Similarly Runnables
                % etc. are always found under InternalBehavior.
                if isa(m3iRootObj, 'Simulink.metamodel.arplatform.component.Component')
                    if isempty(componentAttributes)
                        attribs = Simulink.metamodel.arplatform.ModelFinder.getCompositeAttributes(m3iRootObj);
                        for ii = 1: attribs.size()
                            concreteClasses = autosar.api.getAUTOSARProperties.concreteMetaClassNames(attribs.at(ii).type);
                            for jj = 1 : numel(concreteClasses)
                                componentAttributes = [componentAttributes concreteClasses(jj)]; %#ok<AGROW>
                            end
                        end  
                        componentAttributes = setdiff(componentAttributes, autosar.api.getAUTOSARProperties.UnsupportedCategories);
                    end
                    if isempty(ibAttributes) && m3iRootObj.Behavior.isvalid()
                        attribs = Simulink.metamodel.arplatform.ModelFinder.getCompositeAttributes(m3iRootObj.Behavior);
                        for ii = 1: attribs.size()
                            concreteClasses = autosar.api.getAUTOSARProperties.concreteMetaClassNames(attribs.at(ii).type);
                            for jj = 1 : numel(concreteClasses)
                                ibAttributes = [ibAttributes concreteClasses(jj)]; %#ok<AGROW>
                            end
                        end 
                        ibAttributes = setdiff(ibAttributes, autosar.api.getAUTOSARProperties.UnsupportedCategories);
                    end
                    if any(strcmp(category, componentAttributes))
                        recursiveSearch = false;
                        searchBySuperClass = false;
                    elseif any(strcmp(category, ibAttributes))
                        recursiveSearch = false;
                        searchBySuperClass = false;
                        m3iRootObj = m3iRootObj.Behavior;
                    end
                elseif isa(m3iRootObj, 'Simulink.metamodel.arplatform.behavior.ApplicationComponentBehavior')
                    if isempty(ibAttributes)
                        attribs = Simulink.metamodel.arplatform.ModelFinder.getCompositeAttributes(m3iRootObj);
                        for ii = 1: attribs.size()
                            concreteClasses = autosar.api.getAUTOSARProperties.concreteMetaClassNames(attribs.at(ii).type);
                            for jj = 1 : numel(concreteClasses)
                                ibAttributes = [ibAttributes concreteClasses(jj)]; %#ok<AGROW>
                            end
                        end
                        ibAttributes = setdiff(ibAttributes, autosar.api.getAUTOSARProperties.UnsupportedCategories);
                    end
                    if any(strcmp(category, ibAttributes))
                        recursiveSearch = false;
                        searchBySuperClass = false;
                    end
                end
            else
                m3iRootObj = m3iModel;
            end
            
            if ~isempty(objName)
               % Optimize for MetaClass and name search
               m3iObjSeq = Simulink.metamodel.arplatform.ModelFinder.findObjectByMetaClassAndName(m3iRootObj, metaClass, objName, recursiveSearch);
            else
               m3iObjSeq = autosar.mm.Model.findObjectByMetaClass(m3iRootObj, metaClass, recursiveSearch, searchBySuperClass);    
            end

            paths = cell(1, m3iObjSeq.size());
            for ii=1:m3iObjSeq.size()
                m3iObj = m3iObjSeq.at(ii);
                if strcmp(category, 'ImplementationDataType') && m3iObj.IsApplication
                    paths{ii} = [];
                    continue;
                end
                if strcmp(category, 'ApplicationDataType') && ~m3iObj.IsApplication
                    paths{ii} = [];
                    continue;
                end
                switch pathType
                  case 'FullyQualified'
                    paths{ii} = autosar.api.Utils.getQualifiedName(m3iObj);
                  case 'PartiallyQualified'
                    paths{ii} = autosar.api.getAUTOSARProperties.getPartiallyQualifiedName(m3iObj);
                  otherwise
                    assert(false, 'invalid pathType %s', pathType);
                end
                
                % Check objects actually have propertyNames
                autosar.api.getAUTOSARProperties.validProperties(m3iObj, propertyNames, 'noncomposite');
                
                % Check that propertyValues are same
                for propIdx = 1:length(propertyNames)
                    propName = propertyNames{propIdx};
                    actValue = autosar.api.getAUTOSARProperties.getProperty(m3iObj, propName, pathType);
                    expValue = propertyValues{propIdx};               
                    
                    if isempty(actValue) && ~isempty(expValue)
                        % not same so we're not interested in this path
                            paths{ii} = [];
                            break    
                    elseif isnumeric(actValue) || islogical(actValue)
                        if actValue ~= expValue
                            % not same so we're not interested in this path
                            paths{ii} = [];
                            break
                        end
                    elseif ischar(actValue)
                        if ~strcmp(actValue, expValue)
                            % not same so we're not interested in this path
                            paths{ii} = [];
                            break
                        end
                    else
                        assert(false, 'Cannot compare property %s', propName);
                    end
                end     
            end
            
            emptyIdx = cellfun(@isempty, paths);
            paths(emptyIdx) = [];            
        end

        function [metaClass, validCategory] = getDataTypeMetaClassFromCategory(category)
            import Simulink.metamodel.foundation.*;
            metaClass = [];
            if exist(category, 'class') && any(strcmp(methods(category), 'MetaClass'))
                metaClass = eval(strcat(category, '.MetaClass'));
                
                if isa(metaClass, 'M3I.ImmutableClass')
                    validCategory = true;
                end
            end
        end

        function metaClass = getMetaClassFromCategory(category)
            % Convert user category to a corresponding M3I MetaClass
                
            % import meta-model classes so that we can use just class names, which are
            % unique, rather than qualified class name.
            import Simulink.metamodel.arplatform.behavior.*;
            import Simulink.metamodel.arplatform.common.*;
            import Simulink.metamodel.arplatform.component.*;
            import Simulink.metamodel.arplatform.composition.*;
            import Simulink.metamodel.arplatform.documentation.*;
            import Simulink.metamodel.arplatform.instance.*;
            import Simulink.metamodel.arplatform.interface.*;
            import Simulink.metamodel.arplatform.port.*;
            import Simulink.metamodel.arplatform.variant.*;
            import Simulink.metamodel.types.*;

            % Check that category is a valid M3I MetaClass
            validCategory = false;
            metaClass = [];
            if any(strcmp(category , {'ApplicationDataType', 'ImplementationDataType'}))
                [metaClass, validCategory] = autosar.api.getAUTOSARProperties.getDataTypeMetaClassFromCategory('ValueType');
            end
            if exist(category, 'class') && any(strcmp(methods(category), 'MetaClass'))
               metaClass = eval(strcat(category, '.MetaClass'));
                    
               if isa(metaClass, 'M3I.ImmutableClass')
                   validCategory = true;
               end
            end                    

            if ~validCategory
                validCategories = [autosar.api.getAUTOSARProperties.concreteMetaClassNames(...
                    Simulink.metamodel.arplatform.common.Identifiable.MetaClass), ...
                                   autosar.api.getAUTOSARProperties.abstractMetaClassNames(...
                    Simulink.metamodel.arplatform.common.Identifiable.MetaClass)];
                DAStudio.error('RTW:autosar:apiInvalidCategory', category, '', ...
                    autosar.api.Utils.cell2str(validCategories));
            end
            
        end      
    end
end

% LocalWords:  RPort IB abspathshortname metamodel arplatform DATAOBJ MDLNAME
% LocalWords:  ROOTPATH noncomposite PARAMVALUE ELEMENTPATH PROPERTYNAME sw
% LocalWords:  ADDSRINTERFACE ADDMSINTERFACE QNAME PARENTPATH ADDSWADDRMETHOD
% LocalWords:  RPort IB abspathshortname metamodel arplatform shortname Addr
% LocalWords:  ADDR addPackageableElement MOVEELEMENTS SRCPKG DESTPKG Func
% LocalWords:  Packageable Compu DELETEUNMAPPEDCOMPONENTS CREATENUMERICTYPE
% LocalWords:  COMPUMETHODPATH CREATEENUMERATION autosarstandard Transcation
% LocalWords:  IMPLEMENTATIONDATATYPEPATH CREATESYSTEMCONSTANTS Cutom Nv
% LocalWords:  ADDUNIT ADDPHYSICALDIMENSION Mol DATATYPEPATH Unparented
% LocalWords:  APPLICATIONDATATYPEPATH
