classdef getSimulinkMapping < handle
% autosar.api.getSimulinkMapping API class.
%
% MAPPINGOBJ = autosar.api.getSimulinkMapping(MDLNAME) creates an object
% MAPPINGOBJ that can be used to query and set the Simulink mapping of
% Simulink model MDLNAME.
%
% getSimulinkMapping methods:
%    getDataTransfer   - get the mapping information for a Simulink data transfer line
%    getFunction       - get the mapping information for a Simulink entry point function
%    getInport         - get the mapping information for a Simulink inport
%    getOutport        - get the mapping information for a Simulink outport
%    getFunctionCaller - get the mapping information for a Simulink function caller block
%    getLookupTable    - get the mapping information for a Simulink lookup table
%    mapDataTransfer   - map a Simulink data transfer line
%    mapFunction       - map a Simulink entry point function
%    mapInport         - map an inport
%    mapOutport        - map an outport
%    mapFunctionCaller - map a function caller
%    mapLookupTable    - map a lookup table
%
% See also autosar.api.syncModel autosar.api.create autosar.api.getAUTOSARProperties
    
%   Copyright 2012-2016 The MathWorks, Inc.
    
    properties(Access=private, Transient=true)
        ModelName;
        ChangeLogger;
    end
    
    properties(Constant, Access = private)
        ValidReceiverDAMs = {'ImplicitReceive'
                             'ExplicitReceive'
                             'ExplicitReceiveByVal'
                             'QueuedExplicitReceive'
                             'ErrorStatus'
                             'IsUpdated'
                             'EndToEndRead'};
        ValidSenderDAMs   = {'ImplicitSend'
                             'ImplicitSendByRef'
                             'ExplicitSend'
                             'EndToEndWrite'
                             'QueuedExplicitSend'};
        ValidNvReceiverDAMs = {'ImplicitReceive'
                               'ExplicitReceive'
                               'ExplicitReceiveByVal'};
        ValidNvSenderDAMs   = {'ImplicitSend'
                               'ImplicitSendByRef'
                               'ExplicitSend'};
        ValidParameterAccessModes = {'PortParameter'
                               'PerInstance'
                               'Shared'
                               'ConstantMemory'};
    end
    
    
    methods(Access=public)
        function this = getSimulinkMapping(modelName, changeLogger)            
            this.ModelName = modelName;
            autosar.api.Utils.convertAI2MM(modelName);
            
            if nargin>1
                this.ChangeLogger = changeLogger;
            else
                this.ChangeLogger = [];
            end
        end
           
        function [ARIRVName, ARDataAccessMode] = getDataTransfer(this, SLDataTransferName)
           % [ARIRVNAME, ARDATAACCESSMODE] = GETDATATRANSFER(THIS, SLDATATRANSFERNAME) are
           % the corresponding AUTOSAR InterRunnableVariable ARIRVNAME and
           % DataAccessMode ARDATAACCESSMODE values mapped to the Simulink data
           % transfer line SLDATATRANSFERNAME
           %
           % See also mapDataTransfer

           if ~ischar(SLDataTransferName)
               DAStudio.error('autosarstandard:validation:invalidDataTransferName');
           end
           
           modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
           if autosar.validation.ExportFcnValidator.isTopModelExportFcn(this.ModelName)
               SLSignal = modelMapping.DataTransfers.findobj('SignalName', SLDataTransferName);
            
               if length(SLSignal) ~= 1
                   DAStudio.error('RTW:autosar:invalidMappingDataTransfer', SLDataTransferName);
               end
                    
               ARIRVName = SLSignal.MappedTo.IrvName;
               ARDataAccessMode = SLSignal.MappedTo.IrvAccessMode;
           else                       
              if ~autosar.api.getSimulinkMapping.isValidRateTransitionBlock(SLDataTransferName)
                  DAStudio.error('autosarstandard:validation:invalidDataTransferName');
              end
               
               rtb = modelMapping.RateTransition.findobj('Block',SLDataTransferName);
               if length(rtb) ~= 1
                   DAStudio.error('autosarstandard:validation:invalidRTBDataTransfer', SLDataTransferName);
               end
               
               ARIRVName = rtb.MappedTo.IrvName;
               ARDataAccessMode = rtb.MappedTo.IrvAccessMode;               
           end
        end
        
        function ARRunnableName = getFunction(this, SLFcnName)
        % ARRUNNABLENAME = GETFUNCTION(THIS, SLFCNNAME) is the AUTOSAR 
        % Runnable name ARRUNNABLENAME mapped to Simulink entry point
        % function or Simulink function SLFCNNAME.
        %
        % See also mapFunction
           
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            SLFcnName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLFcnName);
            isExportStyle = autosar.validation.ExportFcnValidator.isTopModelExportFcn(this.ModelName);
            
            isMultiTasking = length(modelMapping.StepFunctions) > 1;
            
            switch SLFcnName
                case 'InitializeFunction'
                    SLFunction = modelMapping.InitializeFunctions(1);
                case 'TerminateFunction'
                    SLFunction = modelMapping.TerminateFunctions(1);
                case 'StepFunction'
                    if isExportStyle
                        DAStudio.error('autosarstandard:validation:invalidMappingFcnForExportFcnModel', SLFcnName);
                    else
                        SLFunction = modelMapping.StepFunctions(1);
                    end
                otherwise
                    resetFunction = autosar.api.getSimulinkMapping.getResetFunction(modelMapping, SLFcnName);
                    if ~isempty(resetFunction)
                        SLFunction = resetFunction;
                    else
                        SLFunction = modelMapping.FcnCallInports.findobj( ...
                            'Block', [modelMapping.Name, '/', SLFcnName]);
                        
                        if ~isMultiTasking && length(SLFunction) ~= 1
                            SLBlock = autosar.api.Utils.getBlockNameFromFunctionName(this.ModelName,...
                                SLFcnName, false);
                            SLFunction = modelMapping.ServerFunctions.findobj('Block', SLBlock);
                        end
                        if isMultiTasking && length(SLFunction) ~= 1
                            tid = regexp(SLFcnName,'^StepFunction(\d+)','tokens');
                            if ~isempty(tid)
                                tid = str2double(tid{1});
                                if ((tid >= 1) && ...
                                        (tid < length(modelMapping.StepFunctions)) )
                                    SLFunction = modelMapping.StepFunctions(tid+1);
                                else
                                    DAStudio.error('autosarstandard:validation:invalidMappingFcnForRateBasedModel', SLFcnName);
                                end
                            else
                                DAStudio.error('autosarstandard:validation:invalidMappingFcnForRateBasedModel', SLFcnName);
                            end
                        end
                        if length(SLFunction) ~= 1
                            DAStudio.error('autosarstandard:validation:invalidMappingFcnForExportFcnModel', SLFcnName);
                        end
                    end
            end
            
            ARRunnableName = SLFunction.MappedTo.Runnable;
        end
        
        
        
        function [ARPortName, ARElementName, ARDataAccessMode] = getInport(this, SLPortName)
           % [ARPORTNAME, ARDATAELEMENTNAME, ARDATAACCESSMODE] = GETINPORT(THIS,
           % SLPORTNAME) are the corresponding AUTOSAR Receiver (or
           % SenderReceiver) port ARPORTNAME, DataElement ARDATAELEMENTNAME and
           % DataAccessMode ARDATAACCESSMODE values mapped to Simulink Inport
           % SLPORTNAME, SLPORTNAME should be the one given by the
           % function get_param(block, 'Name') to ensure no errors
           %
           % See also mapInport
           
               modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
               SLPortName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLPortName);
               
               SLPort = modelMapping.Inports.findobj('Block', [modelMapping.Name, '/', SLPortName]);
           
               if length(SLPort) ~= 1
                   DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'Simulink Port', SLPortName);
               end

               ARPortName = SLPort.MappedTo.Port;
               ARElementName = SLPort.MappedTo.Element;
               ARDataAccessMode = SLPort.MappedTo.DataAccessMode;
           
           end
        
         function [ARPortName, ARElementName, ARDataAccessMode] = getOutport(this, SLPortName)
           % [ARPORTNAME, ARDATAELEMENTNAME, ARDATAACCESSMODE] = GETOUTPORT(THIS,
           % SLPORTNAME) are the corresponding AUTOSAR Sender (or
           % SenderReceiver) port ARPORTNAME, DataElement ARDATAELEMENTNAME and
           % DataAccessMode ARDATAACCESSMODE values mapped to Simulink Outport
           % SLPORTNAME, SLPORTNAME should be the one given by the
           % function get_param(block, 'Name') to ensure no errors
           %
           % See also mapOutport

               modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
               SLPortName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLPortName);
               SLPort = modelMapping.Outports.findobj('Block', [modelMapping.Name, '/', SLPortName]);
           
               if length(SLPort) ~= 1
                   DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'Simulink Port', SLPortName);
               end

               ARPortName = SLPort.MappedTo.Port;
               ARElementName = SLPort.MappedTo.Element;
               ARDataAccessMode = SLPort.MappedTo.DataAccessMode;
                          
         end
         
         function [ARPortName, AROperationName] = getFunctionCaller(this, SLFcnName)
             % [ARPORTNAME, AROPERATIONNAME] = GETFUNCTIONCALLER(THIS,
             % SLFCNNAME) are the corresponding AUTOSAR Client
             % port ARPORTNAME, Operation AROPERATIONNAME values
             % mapped to Simulink function caller SLFCNNAME.
             % 
             % See also mapFunctionCaller
             
             modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
 	     SLFcnName = strrep(SLFcnName, sprintf('\n'), ' ');
             SLBlock = autosar.api.Utils.getBlockNameFromFunctionName(this.ModelName,...
                 SLFcnName, true);
             SLPort = modelMapping.FunctionCallers.findobj('Block', SLBlock);
             
             if length(SLPort) ~= 1
                 DAStudio.error('RTW:autosar:invalidFunctionCallerBlock', SLFcnName);
             end
             
             ARPortName = SLPort.MappedTo.ClientPort;
             AROperationName = SLPort.MappedTo.Operation;
             
         end
         
        function mapDataTransfer(this, SLDataTransferName, ARIRVName, ARDataAccessMode)
        % MAPDATATRANSFER(THIS, SLDATATRANSFERNAME, ARIRVNAME, ARDATAACCESSMODE) maps
        % Simulink data transfer line SLDATATRANSFERNAME to
        % InterRunnableVariable ARIRVNAME. DataAccessMODE ARDATAACCESSMODE
        % can be one of 'Implicit', 'Explicit'
        %
        % See also getDataTransfer
           
            if ~ischar(SLDataTransferName)
                DAStudio.error('autosarstandard:validation:invalidDataTransferName');
            end
        
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            dataObj = autosar.api.getAUTOSARProperties(this.ModelName, true);            
                       
            if autosar.validation.ExportFcnValidator.isTopModelExportFcn(this.ModelName)
                
                SLSignal = modelMapping.DataTransfers.findobj('SignalName', SLDataTransferName);
                
                if length(SLSignal) ~= 1
                    DAStudio.error('RTW:autosar:invalidMappingDataTransfer', SLDataTransferName);
                end
                componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
                ARIRVPaths = dataObj.find(componentQualifiedName, 'IrvData', ...
                    'Name', ARIRVName, 'PathType', 'FullyQualified');
                if length(ARIRVPaths) ~= 1
                    DAStudio.error('RTW:autosar:invalidMappingDataTransferIrv', ...
                        SLDataTransferName, ARIRVName);
                end
                
                
                if ~isempty(this.ChangeLogger)
                    if ~strcmp(SLSignal.MappedTo.IrvName, ARIRVName) &&...
                        ~isempty(SLSignal.MappedTo.IrvName)
                        this.ChangeLogger.logModification('Automatic', 'IRVName', ...
                            'Simulink data transfer', ...
                            SLDataTransferName, ...
                            SLSignal.MappedTo.IrvName, ...
                            ARIRVName);
                    end
                    
                    if ~strcmp(SLSignal.MappedTo.IrvAccessMode, ARDataAccessMode)
                        this.ChangeLogger.logModification('Automatic', 'DataAccessMode', ...
                            'Simulink data transfer', ...
                            SLDataTransferName, ...
                            SLSignal.MappedTo.IrvAccessMode, ...
                            ARDataAccessMode);
                    end
                end               
                
                SLSignal.mapInterRunnableVariable(SLDataTransferName, ARIRVName, ARDataAccessMode, '');                
            else
                
                if ~autosar.api.getSimulinkMapping.isValidRateTransitionBlock(SLDataTransferName)
                  DAStudio.error('autosarstandard:validation:invalidDataTransferName');
                end
                
                if ~(strcmp(get_param(SLDataTransferName,'Integrity'), 'on') && ...
                       strcmp(get_param(SLDataTransferName,'Deterministic'), 'off'))
                   DAStudio.error(...
                       'autosarstandard:validation:RateTranBlkForIRV',...
                       SLDataTransferName);
                end
                
                rtb = modelMapping.RateTransition.findobj('Block',SLDataTransferName);
                if length(rtb) ~= 1
                    DAStudio.error('autosarstandard:validation:invalidRTBDataTransfer', SLDataTransferName);
                end
                
                componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
                ARIRVPaths = dataObj.find(componentQualifiedName, 'IrvData', ...
                    'Name', ARIRVName, 'PathType', 'FullyQualified');
                if length(ARIRVPaths) ~= 1
                    DAStudio.error('autosarstandard:validation:IRVNotExistForDataTransfer', ...
                        SLDataTransferName, ARIRVName);
                end
                                
                if ~isempty(this.ChangeLogger)
                    if ~strcmp(rtb.MappedTo.IrvName, ARIRVName) &&...
                        ~isempty(rtb.MappedTo.IrvName)
                        this.ChangeLogger.logModification('Automatic', 'IRVName', ...
                            'Simulink data transfer', ...
                            SLDataTransferName, ...
                            rtb.MappedTo.IrvName, ...
                            ARIRVName);
                    end
                    
                    if ~strcmp(rtb.MappedTo.IrvAccessMode, ARDataAccessMode)
                        this.ChangeLogger.logModification('Automatic', 'DataAccessMode', ...
                            'Simulink data transfer', ...
                            SLDataTransferName, ...
                            rtb.MappedTo.IrvAccessMode, ...
                            ARDataAccessMode);
                    end
                end               
                
                rtb.mapInterRunnableVariable(ARIRVName, ARDataAccessMode, '');                 
            end
            
        end
        
        function mapInport(this, SLPortName, ARPortName, ARDataName, ARDataAccessModeStr)
            % MAPINPORT(THIS, SLPORTNAME, ARPORTNAME, ARDATAELEMENTNAME, ARDATAACCESSMODE)
            % maps Simulink Inport SLPORTNAME to DataElement ARDATAELEMENTNAME
            % of AUTOSAR Receiver (or SenderReceiver) port
            % ARPORTNAME. SLPORTNAME should be the one given by the
            % function get_param(block, 'Name') to ensure no errors
            % DataAccessMode ARDATAACCESSMODESTR can be one of
            % 'ImplicitReceive', 'ExplicitReceive', 'ExplicitReceiveByVal', 
            % 'QueuedExplicitReceive', 'ErrorStatus', 'IsUpdated', 
            % 'EndToEndRead'.
            %
            % See also getInport
            
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            dataObj = autosar.api.getAUTOSARProperties(this.ModelName, true);
            SLPortName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLPortName);
            SLPort = modelMapping.Inports.findobj('Block', [modelMapping.Name, '/', SLPortName]);
           
            % SLPort Check
            if length(SLPort) ~= 1
                    DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'Simulink Port', SLPortName);
            end

            % ARPort Check
            if strcmp(ARDataAccessModeStr, 'ModeReceive')
                portType = 'ModeReceiverPort';
            else
                portType = 'DataReceiverPort';
            end
            componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
            ARPortPaths = dataObj.find(componentQualifiedName, portType, ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
            
            % May be trying to map a SenderReceiverPort
            if isempty(ARPortPaths) && strcmp(portType, 'DataReceiverPort')
                 ARPortPaths = dataObj.find(componentQualifiedName, 'DataSenderReceiverPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
                 if ~isempty(ARPortPaths)
                     % Found a DataSenderReceiverPort
                     portType = 'DataSenderReceiverPort';
                 end
            end
                
            % May be trying to map a NvDataReceiverPort
            if isempty(ARPortPaths) && strcmp(portType, 'DataReceiverPort')
                 ARPortPaths = dataObj.find(componentQualifiedName, 'NvDataReceiverPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
                 if ~isempty(ARPortPaths)
                     % Found a NvDataReceiverPort
                     portType = 'NvDataReceiverPort';
                 end
            end            
            
            % May be trying to map a NvDataSenderReceiverPort
            if isempty(ARPortPaths) && strcmp(portType, 'DataReceiverPort')
                 ARPortPaths = dataObj.find(componentQualifiedName, 'NvDataSenderReceiverPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
                 if ~isempty(ARPortPaths)
                     % Found a NvDataSenderReceiverPort
                     portType = 'NvDataSenderReceiverPort';
                 end
            end            
            
            if isempty(ARPortPaths) && strcmp(portType, 'ModeReceiverPort')
                % After 3.x import SenderReceiverInterface referenced by a DataReceiverPort may have ModeDeclarationGroupElement
                ARPortPaths = dataObj.find(componentQualifiedName, 'DataReceiverPort', ...
                    'Name', ARPortName, 'PathType', 'FullyQualified');
            end
            
            if length(ARPortPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, portType, ARPortName);
            end
            
            % ARDataElement Check           
            ifPath = dataObj.get(ARPortPaths{1}, 'Interface', 'PathType', 'FullyQualified');
            if strcmp(ARDataAccessModeStr, 'ModeReceive')
                metaModelClassName = 'ModeDeclarationGroupElement';
            else
                metaModelClassName = 'FlowData';
            end
            ARDataElementPaths = dataObj.find(ifPath, metaModelClassName, ...
                'Name', ARDataName, 'PathType', 'FullyQualified');
            if isempty(ifPath) || length(ARDataElementPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'DataElement', ARDataName);
            end
            
            switch portType
              case 'DataReceiverPort'
                validDataAccessModes = this.ValidReceiverDAMs;
              case 'DataSenderReceiverPort'
                validDataAccessModes = {'ImplicitReceive', ...
                                    'ExplicitReceive', ...
                                    'ExplicitReceiveByVal', ...
                                    'ErrorStatus', ...
                                    'IsUpdated', ...
                                    'EndToEndRead'};
              case {'NvDataReceiverPort', 'NvDataSenderReceiverPort'}
                validDataAccessModes = this.ValidNvReceiverDAMs;
              case 'ModeReceiverPort'
                validDataAccessModes = {'ModeReceive'};
              otherwise
                assert(false, 'Did not recognize portType %s', portType);
            end
                
            
            % ARDataAccessModeStr Check
            if ~any(strcmp(ARDataAccessModeStr, validDataAccessModes))
                validDataAccessModesStr = autosar.api.Utils.cell2str(validDataAccessModes);
                DAStudio.error('RTW:autosar:invalidMappingDAM', ARDataAccessModeStr, SLPortName, validDataAccessModesStr);
            end

            if ~isempty(this.ChangeLogger)
                if ~strcmp(SLPort.MappedTo.Port, ARPortName)
                    this.ChangeLogger.logModification('Automatic', 'ARPortName', ...
                                                      'Simulink inport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.Port, ...
                                                      ARPortName);
                end
                    
                if ~strcmp(SLPort.MappedTo.Element, ARDataName)
                    this.ChangeLogger.logModification('Automatic', 'ARElementName', ...
                                                      'Simulink inport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.Element, ...
                                                      ARDataName);
                end
                
                if ~strcmp(SLPort.MappedTo.DataAccessMode, ARDataAccessModeStr)
                    this.ChangeLogger.logModification('Automatic', 'DataAccessMode', ...
                                                      'Simulink inport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.DataAccessMode, ...
                                                      ARDataAccessModeStr);
                end
            end
            
            SLPort.mapPortElement(ARPortName, ARDataName, ARDataAccessModeStr, '');

            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
        
        function mapOutport(this, SLPortName, ARPortName, ARDataName, ARDataAccessModeStr)
            % MAPOUTPORT(THIS, SLPORTNAME, ARPORTNAME, ARDATAELEMENTNAME, ARDATAACCESSMODE)
            % maps Simulink Outport SLPORTNAME to DataElement ARDATAELEMENTNAME
            % of AUTOSAR Sender (or SenderReceiver) port
            % ARPORTNAME. SLPORTNAME should be the one given by the
            % function get_param(block, 'Name') to ensure no errors
            % DataAccessMode ARDATAACCESSMODESTR can be one of
            % 'ImplicitSend', 'ImplicitSendByRef', 'ExplicitSend', 
            % 'EndToEndWrite'.
            %
            % See also getOutport
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            dataObj = autosar.api.getAUTOSARProperties(this.ModelName, true);
            SLPortName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLPortName);
            SLPort = modelMapping.Outports.findobj('Block', [modelMapping.Name, '/', SLPortName]);
            
            if length(SLPort) ~= 1                  
                DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'Simulink Port', SLPortName);               
            end

            if strcmp(ARDataAccessModeStr, 'ModeSend')
                portType = 'ModeSenderPort';
            else
                portType = 'DataSenderPort';
            end
            
            % ARPort Check
            componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
            ARPortPaths = dataObj.find(componentQualifiedName, portType, ...
                'Name', ARPortName, 'PathType', 'FullyQualified');
            
            % May be trying to map a SenderReceiverPort
            if isempty(ARPortPaths)
                 ARPortPaths = dataObj.find(componentQualifiedName, 'DataSenderReceiverPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
            end
            
            % May be trying to map a NvDataSenderPort
            if isempty(ARPortPaths)
                 ARPortPaths = dataObj.find(componentQualifiedName, 'NvDataSenderPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
                 if ~isempty(ARPortPaths)
                     % Found a NvDataSenderPort
                     portType = 'NvDataSenderPort';
                 end
            end     
            
            % May be trying to map a NvDataSenderReceiverPort
            if isempty(ARPortPaths)
                 ARPortPaths = dataObj.find(componentQualifiedName, 'NvDataSenderReceiverPort', ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');
                 if ~isempty(ARPortPaths)
                     % Found a NvDataSenderPort
                     portType = 'NvDataSenderPort';
                 end
            end
            
            if isempty(ARPortPaths) && strcmp(portType, 'ModeSenderPort')
                % After 3.x import SenderReceiverInterface referenced by
                % a DataSenderPort may have ModeDeclarationGroupElement??
                
                ARPortPaths = dataObj.find(componentQualifiedName, 'DataSenderPort', ...
                    'Name', ARPortName, 'PathType', 'FullyQualified');
            end
            
            if length(ARPortPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'DataSenderPort', ARPortName);
            end
            
            % ARDataElement Check           
            ifPath = dataObj.get(ARPortPaths{1}, 'Interface', 'PathType', 'FullyQualified');
            if strcmp(ARDataAccessModeStr, 'ModeSend')
                metaModelClassName = 'ModeDeclarationGroupElement';
            else
                metaModelClassName = 'FlowData';
            end            
            ARDataElementPaths = dataObj.find(ifPath, metaModelClassName, ...
                'Name', ARDataName, 'PathType', 'FullyQualified');
            if isempty(ifPath) || length(ARDataElementPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidMappingPort', SLPortName, 'DataElement', ARDataName);
            end
            
            % ARDataAccessModeStr Check
            switch portType
              case 'DataSenderPort'
                validDataAccessModes = this.ValidSenderDAMs;
              case 'NvDataSenderPort'
                validDataAccessModes = this.ValidNvSenderDAMs;
              case 'ModeSenderPort'
                validDataAccessModes = {'ModeSend'};                
              otherwise
                assert(false, 'Did not recognize portType %s', portType);
            end
            
            
            % ARDataAccessModeStr Check
            if ~any(strcmp(ARDataAccessModeStr, validDataAccessModes))
                validDataAccessModesStr = autosar.api.Utils.cell2str(validDataAccessModes);
                DAStudio.error('RTW:autosar:invalidMappingDAM', ARDataAccessModeStr, SLPortName, validDataAccessModesStr);
            end
            
            if ~isempty(this.ChangeLogger)
                if ~strcmp(SLPort.MappedTo.Port, ARPortName)
                    this.ChangeLogger.logModification('Automatic', 'ARPortName', ...
                                                      'Simulink outport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.Port, ...
                                                      ARPortName);
                end
                    
                if ~strcmp(SLPort.MappedTo.Element, ARDataName)
                    this.ChangeLogger.logModification('Automatic', 'ARElementName', ...
                                                      'Simulink outport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.Element, ...
                                                      ARDataName);
                end
                
                % ------- Keeping Implicit send selection on update -------
                % In the AUTOSAR ARXML both ImplicitSend and 
                % ImplicitSendByRef are the expressed in the same way. When
                % updating we want to keep the previous user selection in
                % this case.
                implicitSendAccessModes = {'ImplicitSend', 'ImplicitSendByRef'};
                currIsImplicitSendAm = ismember(SLPort.MappedTo.DataAccessMode, ...
                                                implicitSendAccessModes);
                newIsImplicitSendAm = ismember(ARDataAccessModeStr, ...
                                               implicitSendAccessModes);
                if (currIsImplicitSendAm && newIsImplicitSendAm)
                    ARDataAccessModeStr = SLPort.MappedTo.DataAccessMode;
                end
                % ---------------------------------------------------------
                
                if ~strcmp(SLPort.MappedTo.DataAccessMode, ARDataAccessModeStr)
                    this.ChangeLogger.logModification('Automatic', 'DataAccessMode', ...
                                                      'Simulink outport', ...
                                                      SLPort.Block, ...
                                                      SLPort.MappedTo.DataAccessMode, ...
                                                      ARDataAccessModeStr);
                end
            end
            
            SLPort.mapPortElement(ARPortName, ARDataName, ARDataAccessModeStr, '');
            
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
        
        
        function mapFunction(this, SLFcnName, ARRunnableName)
            % MAPFUNCTION(THIS, SLFCNNAME, ARRUNNABLENAME) maps Simulink entry point
            % function or Simulink function SLFCNNAME to AUTOSAR Runnable 
            % ARRUNNABLENAME.
            %
            % See also getFunction
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            % ARRunnableName cannot be empty.
            if isempty(ARRunnableName)
                DAStudio.error('RTW:autosar:invalidRunnableName',...
                    ARRunnableName);
            end
            
            SLFcnName = autosar.api.getSimulinkMapping.escapeSimulinkName(SLFcnName);
            isExportStyle = autosar.validation.ExportFcnValidator.isTopModelExportFcn(this.ModelName);
            isMultiTasking = length(modelMapping.StepFunctions) > 1;
            
            mappingObj = [];
            switch SLFcnName
                case 'InitializeFunction'
                    mappingObj = modelMapping.InitializeFunctions(1);
                case 'TerminateFunction'
                    mappingObj = modelMapping.TerminateFunctions(1);
                case 'StepFunction'
                    if ~isExportStyle
                        mappingObj = modelMapping.StepFunctions(1);
                    end
                otherwise
                    SLFcnCallInport = modelMapping.FcnCallInports.findobj( ...
                        'Block', [modelMapping.Name, '/', SLFcnName]);
                    if length(SLFcnCallInport) == 1
                        mappingObj = SLFcnCallInport;
                    end
                    if ~isMultiTasking && isempty(mappingObj)
                        SLBlock = autosar.api.Utils.getBlockNameFromFunctionName(this.ModelName,...
                            SLFcnName, false);
                        SLServerFunction = modelMapping.ServerFunctions.findobj( ...
                            'Block', SLBlock);
                        if length(SLServerFunction) == 1
                            mappingObj = SLServerFunction;
                        end
                    end
                    if isempty(mappingObj)
                        mappingObj = autosar.api.getSimulinkMapping.getResetFunction(modelMapping, SLFcnName);
                    end
                    if isMultiTasking && isempty(mappingObj)
                        tid = regexp(SLFcnName,'^StepFunction(\d+)','tokens');
                        if ~isempty(tid)
                            tid = str2double(tid{1});
                            if ((tid >= 1) && ...
                                    (tid < length(modelMapping.StepFunctions)) )
                                mappingObj = modelMapping.StepFunctions(tid+1);
                            else
                                DAStudio.error('autosarstandard:validation:invalidMappingFcnForRateBasedModel', SLFcnName);
                            end
                        else
                            DAStudio.error('autosarstandard:validation:invalidMappingFcnForRateBasedModel', SLFcnName);
                        end
                    end
            end
            
            if isempty(mappingObj)
                DAStudio.error('autosarstandard:validation:invalidMappingFcnForExportFcnModel', SLFcnName);
            end
            
            % Call function in utils to do the mapping
            autosar.api.Utils.mapFunction(this.ModelName,...
                mappingObj, ARRunnableName);
            
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
        
        function mapFunctionCaller(this, SLFcnName, ARPortName, AROperationName)
            % MAPFUNCTIONCALLER(THIS, SLFCNNAME, ARPORTNAME, AROPERATIONNAME)
            % maps Simulink Function caller SLFCNNAME to Operation AROPERATIONNAME
            % of AUTOSAR Client port ARPORTNAME.
            %
            % See also getFunctionCaller
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            dataObj = autosar.api.getAUTOSARProperties(this.ModelName, true);
                       
            SLFcnName = strrep(SLFcnName, sprintf('\n'), ' ');
            SLBlock = autosar.api.Utils.getBlockNameFromFunctionName(this.ModelName,...
                            SLFcnName, true);
            SLBlockMap = modelMapping.FunctionCallers.findobj('Block', SLBlock);              
            
            % SLBlockMap Check
            if length(SLBlockMap) ~= 1
                DAStudio.error('RTW:autosar:invalidCallerMapping', SLFcnName, 'function caller', SLFcnName);
            end

            % ARPort Check
            portType = 'ClientPort';
            componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
            ARPortPaths = dataObj.find(componentQualifiedName, portType, ...
                 'Name', ARPortName, 'PathType', 'FullyQualified');            
            
            if length(ARPortPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidCallerMapping', SLFcnName, portType, ARPortName);
            end
            
            % AROperationName Check           
            ifPath = dataObj.get(ARPortPaths{1}, 'Interface', 'PathType', 'FullyQualified');
            metaModelClassName = 'Operation';
            AROperationPaths = dataObj.find(ifPath, metaModelClassName, ...
                'Name', AROperationName, 'PathType', 'FullyQualified');
            if isempty(ifPath) || length(AROperationPaths) ~= 1
                DAStudio.error('RTW:autosar:invalidCallerMapping', SLFcnName, 'Operation', AROperationName);
            end                
            

            if ~isempty(this.ChangeLogger)
                if ~strcmp(SLBlockMap.MappedTo.ClientPort, ARPortName)
                    this.ChangeLogger.logModification('Automatic', 'ARPortName', ...
                                                      'Simulink Function Caller block', ...
                                                      SLBlockMap.Block, ...
                                                      SLBlockMap.MappedTo.ClientPort, ...
                                                      ARPortName);
                end
                    
                if ~strcmp(SLBlockMap.MappedTo.Operation, AROperationName)
                    this.ChangeLogger.logModification('Automatic', 'AROperationName', ...
                                                      'Simulink Function Caller block', ...
                                                      SLBlockMap.Block, ...
                                                      SLBlockMap.MappedTo.Operation, ...
                                                      AROperationName);
                end
            end
            for index=1:length(modelMapping.FunctionCallers)
                if strcmp(get_param(modelMapping.FunctionCallers(index).Block, 'FunctionPrototype'),...
                        get_param(SLBlockMap.Block, 'FunctionPrototype'))
                    % Call function in utils to do the mapping
                    autosar.api.Utils.mapCaller(this.ModelName, modelMapping.FunctionCallers(index), ...
                        ARPortName, AROperationName);
                end
            end

            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
        
        function [ARParameterAccessMode, ARPortName, ARParameterData] = getLookupTable(this, SLParam)
            % [ARPARAMETERACCESSMODE, ARPORTNAME, ARPARAMETERDATA] = GETLOOKUPTABLE(THIS,
            % SLPARAM) are the corresponding ParameterAccessMode
            % ARPARAMETERACCESSMODE, AUTOSAR Parameter Receiver port
            % ARPORTNAME and Parameter Data ARPARAMETERDATA values mapped
            % to Simulink Lookup Table SLPARAM.
            %
            % See also mapLookupTable
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            
            SLLut = modelMapping.LookupTables.findobj('LookupTableName', SLParam);
            
            if length(SLLut) ~= 1
                DAStudio.error('autosarstandard:api:invalidMapping', 'Lookup Table',...
                    SLParam, 'Simulink Lookup Table', SLParam);
            end
            
            ARParameterAccessMode = SLLut.MappedTo.ParameterAccessMode;
            ARPortName = SLLut.MappedTo.Port;
            ARParameterData = SLLut.MappedTo.Parameter;
        end
        
        function mapLookupTable(this, SLParam, ARParameterAccessMode, ARPortName, ARParameterData)
            % MAPLOOKUPTABLE(THIS, SLPARAM, ARPARAMETERACCESSMODE, ARPORTNAME, ARPARAMETERDATA)
            % maps Simulink Lookup Table SLPARAM to ParameterAccessMode
            % ARPARAMETERACCESSMODE, AUTOSAR Parameter Receiver Port
            % ARPORTNAME and Parameter Data ARPARAMETERDATA.
            % ParameterAccessMode ARPARAMETERACCESSMODE can be one of
            % PortParameter, Shared, PerInstance and Const.
            %
            % Examples: 
            % Map Simulink lookup table L_4_single to AUTOSAR Parameter
            % with ParameterAccessMode Shared.
            %
            % mappingObj = autosar.api.getSimulinkMapping(modelName);
            % mappingObj.mapLookupTable('L_4_single', 'Shared', '',...
            %   'L_4_single_ar');
            %
            % Map Simulink lookup table L_4_single to AUTOSAR Parameter
            % to AUTOSAR Parameter Receiver Port paramRPort and Parameter
            % paramElem1.
            %
            % mappingObj = autosar.api.getSimulinkMapping(modelName);
            % mappingObj.mapLookupTable('L_4_single', 'PortParameter',...
            %   'paramRPort', 'paramElem1');
            %
            % See also getLookupTable
            
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            dataObj = autosar.api.getAUTOSARProperties(this.ModelName, true);
            
            SLLUTMap = modelMapping.LookupTables.findobj('LookupTableName', SLParam);
            
            % SL LookupTable Check
            if length(SLLUTMap) ~= 1
                DAStudio.error('autosarstandard:api:invalidMapping', 'Lookup Table',...
                    SLParam, 'Simulink Lookup Table', SLParam);
            end
            
            if strcmp(ARParameterAccessMode, 'PortParameter')
                % ARPort Check
                portType = 'ParameterReceiverPort';
                componentQualifiedName = dataObj.get('XmlOptions', 'ComponentQualifiedName');
                ARPortPaths = dataObj.find(componentQualifiedName, portType, ...
                    'Name', ARPortName, 'PathType', 'FullyQualified');
                
                if length(ARPortPaths) ~= 1
                    DAStudio.error('autosarstandard:api:invalidMapping', 'Lookup Table', ...
                        SLParam, portType, ARPortName);
                end
                
                % ARParameterName Check
                ifPath = dataObj.get(ARPortPaths{1}, 'Interface', 'PathType', 'FullyQualified');
                metaModelClassName = 'ParameterData';
                ARParameterDataPaths = dataObj.find(ifPath, metaModelClassName, ...
                    'Name', ARParameterData, 'PathType', 'FullyQualified');
                if isempty(ifPath) || length(ARParameterDataPaths) ~= 1
                    DAStudio.error('autosarstandard:api:invalidMapping', 'Lookup Table', ...
                        SLParam, 'Parameter Data', ARParameterData);
                    
                end
            end
            % ARParameterAccessMode Check
            if ~any(strcmp(ARParameterAccessMode, this.ValidParameterAccessModes))
                validParameterDataRolesStr = autosar.api.Utils.cell2str(this.ValidParameterAccessModes);
                DAStudio.error('autosarstandard:api:invalidParameterMappingRole',...
                    ARParameterAccessMode, 'Simulink Lookup Table', SLLUTMap.LookupTableName,...
                    validParameterDataRolesStr);
            end
            SLLUTMap.mapLookupTable(SLLUTMap.LookupTableName, ARParameterAccessMode,...
                ARPortName, ARParameterData, '');
            % Dirty the model
            set_param(this.ModelName,'Dirty','on');
        end
    end
    
    methods(Hidden)
        
        function mapPort(this, SLPortName, ARPortName, ARDataName, ARDataAccessModeStr)
            % MAPPORT(THIS, SLPORTNAME, ARPORTNAME, ARDATAELEMENTNAME, ARDATAACCESSMODE)
            % maps Simulink port SLPORTNAME to DataElement ARDATAELEMENTNAME
            % of AUTOSAR Sender/Receiver (or SenderReceiver) port
            % ARPORTNAME. SLPORTNAME should be the one given by the
            % function get_param(block, 'Name') to ensure no errors
            % 
            % See also mapInport, mapOutport
            modelMapping = autosar.api.Utils.modelMapping(this.ModelName);
            SLPortName2 = autosar.api.getSimulinkMapping.escapeSimulinkName(SLPortName);
            SLInport = modelMapping.Inports.findobj('Block', [modelMapping.Name, '/', SLPortName2]);

            if length(SLInport) == 1
                this.mapInport(SLPortName, ARPortName,...
                    ARDataName, ARDataAccessModeStr);
            else
                this.mapOutport(SLPortName, ARPortName,...
                    ARDataName, ARDataAccessModeStr);
            end
        end
    
    end

    methods(Static, Hidden)
        
        function validNvReceiverDAMs = getValidNvReceiverDAMs()
            % VALIDNVRECEIVERDAMS = GETVALIDNVRECEIVERDAMS returns a cell-array of valid
            % NvReceiverPort Data Access Modes.
            validNvReceiverDAMs = autosar.api.getSimulinkMapping.ValidNvReceiverDAMs;
        end
        
        function validNvSenderDAMs = getValidNvSenderDAMs()
            % VALIDNVSENDERDAMS = GETVALIDNVSENDERDAMS returns a cell-array of valid
            % NvSenderPort Data Access Modes.
            validNvSenderDAMs = autosar.api.getSimulinkMapping.ValidNvSenderDAMs;
        end
    
    end
    
    methods(Static, Access=private)
        function res = escapeSimulinkName(pathStr)
        % Convert new line to space as this is what
        % BlockMapping.getBlockPath() does, and change / into // as this is
        % what getFullName() does when a port name is obtained 
           pathStr = strrep(pathStr, sprintf('\n'), ' ');
           res = strrep(pathStr, '/', '//');
        end      
        
        function isValid = isValidRateTransitionBlock(blkPath)
            % Check if input string is a valid rate transition block
            isValid = false;
            try
               if Simulink.ID.isValid(Simulink.ID.getSID(blkPath))
                   if strcmp(get_param(blkPath,'BlockType'), 'RateTransition')
                        isValid = true;                      
                   end
               end
            catch ex %#ok<NASGU>
            end
        end
        
        function resetFunction = getResetFunction(modelMapping, SLResetFcnName)
            for idx = 1:length(modelMapping.ResetFunctions)
                currResetFcn = modelMapping.ResetFunctions(idx);
                if strcmp(SLResetFcnName, currResetFcn.Name)
                    resetFunction = currResetFcn;
                    return;
                end
            end
            resetFunction = [];
            return;
        end        
    end
    
end

% LocalWords:  MAPPINGOBJ MDLNAME ARIRVNAME ARDATAACCESSMODE GETDATATRANSFER
% LocalWords:  SLDATATRANSFERNAME ARRUNNABLENAME GETFUNCTION SLFCNNAME utils
% LocalWords:  ARPORTNAME ARDATAELEMENTNAME GETINPORT SLPORTNAME GETOUTPORT RTB
% LocalWords:  MAPDATATRANSFER MAPINPORT ARDATAACCESSMODESTR MAPOUTPORT MAPPORT
% LocalWords:  MAPFUNCTION AROPERATIONNAME GETFUNCTIONCALLER MAPFUNCTIONCALLER
% LocalWords:  ARCOMPUMETHODNAME SLDATATYPENAME Compu MAPDATATYPE Nv RPort
% LocalWords:  autosarstandard ARLOOKUPTABLENAME ARPARAMETERACCESSMODE GETLOOKUPTABLE
% LocalWords:  ARPARAMETERDATANAME MAPLOOKUPTABLE SLPARAM ARPARAMETERDATA
% LocalWords:  VALIDNVRECEIVERDAMS GETVALIDNVRECEIVERDAMS VALIDNVSENDERDAMS
% LocalWords:  GETVALIDNVSENDERDAMS
