function [modelH, success] = createComponentAsModel(this, ComponentName, varargin)
%CREATECOMPONENTASMODEL creates an AUTOSAR Atomic-Software-Component as a 
%   Simulink model.
%
%   createComponentAsModel(IMPORTEROBJ, 'COMPONENTNAME') creates the Simulink
%   model corresponding to the AUTOSAR Atomic-Software-Component 'COMPONENT' 
%   described in the XML file imported by the arxml.importer object IMPORTEROBJ.
%
%   Usage:
%      [modelH, success] = createComponentAsModel(importerObj,...
%                                                 componentName, varargin)
%
%   Inputs:
%      importerObj:   arxml.importer object
%      componentName: absolute Short-Name path of the Atomic-Software-Component
%      varargin:      Parameter/Value pairs. Supported parameters are:
%
%         ModelPeriodicRunnablesAs: AtomicSubsystem (default), FunctionCallSubsystem,
%         or Auto. By default, createComponentAsModel imports AUTOSAR
%         periodic runnables and models them as atomic subsystems with
%         periodic rates. If conditions prevent use of atomic subsystems,
%         an error is thrown. To model periodic runnables as function-call
%         subsystems with periodic rates, specify FunctionCallSubsystem. If
%         you specify Auto, createComponentAsModel attempts to model
%         periodic runnables as atomic subsystems, if possible. If
%         conditions prevent use of atomic subsystems, it will model them
%         as function-call subsystems.
%
%         DataDictionary: String specifying a Simulink data dictionary into
%         which to import data objects corresponding to AUTOSAR data types in
%         the XML file. The model becomes associated with the specified data
%         dictionary.
% 
%         InitializationRunnable: String specifying the name of an existing
%         runnable as the initialization runnable for the component.
%
%         PredefinedVariant: String specifying the path to a 
%         PredefinedVariant defined in the AUTOSAR XML file. For example, 
%         '/pkg/body/Variants/Senior'.
%
%         SystemConstValueSets: Cell array of strings specifying paths to
%         one or more SwSystemconstantValueSets defined in the AUTOSAR XML 
%         file. For example, {'/pkg/body/SystemConstantValues/A',
%         '/pkg/body/SystemConstantValues/B'}.
%
%   Outputs:
%      modelH:  Model handle.
%      success: It is true if this function is successful. Otherwise, 
%               it is false.
%
%   See also arxml.importer.getComponentNames
%      arxml.importer.getFile arxml.importer.setFile 
%      arxml.importer.getDependencies arxml.importer.setDependencies

%   Copyright 2007-2016 The MathWorks, Inc.

  
success = false; %#ok
modelH  = -1;    %#ok

% Create an instance of the inputParser class.
argParser = inputParser;   

% Add arguments to the schema
argParser.addRequired('this', @(x) isa(x, class(x)));
argParser.addRequired('ComponentName', @ischar);
argParser.addParameter('ModelPeriodicRunnablesAs', 'AtomicSubsystem',...
    @(x)any(strcmp(x,{'Auto','AtomicSubsystem','FunctionCallSubsystem'})));
argParser.addParameter('InitializationRunnable', '', @(x)(ischar(x)));
argParser.addParameter('DataDictionary', '', @(x)(ischar(x)));
argParser.addParameter('PredefinedVariant', '', @(x)(ischar(x)));
argParser.addParameter('SystemConstValueSets', '', @(x)(iscell(x)));

% parameters that are not intended to be customer visible
argParser.addParameter('CreateSimulinkObject', true, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('NameConflictAction', 'overwrite', @(x)any(strcmpi(x,{'overwrite','makenameunique','error'})));
argParser.addParameter('AutoSave', false, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('CreateInternalBehavior', false, @(x)(islogical(x)||(x==1)||(x==0)));
argParser.addParameter('OpenModelAfterCreation', true, @(x)(islogical(x)));

% Read and validate each argument
argParser.parse(this, ComponentName, varargin{:});

% Check if the DataDictionary exists
autosar.mm.mm2sl.utils.checkAndCreateDD(argParser.Results.DataDictionary)
% Create the component
[modelH, success] = p_create_slcomponent(this, argParser);

% LocalWords:  IMPORTEROBJ COMPONENTNAME makenameunique BSP interrunnable Sw
% LocalWords:  Systemconstant
